# -*- coding: utf-8 -*-
# Django settings for sandbox project.

import os, sys
from django.core.urlresolvers import reverse_lazy, reverse

import environ

# set default values and casting
env = environ.Env(DEBUG=(bool, False),
                  CELERY_ALWAYS_EAGER=(bool, False),
                  )

# Django settings for server project.
DEBUG = env('DEBUG') # False if not in os.environ
TEMPLATE_DEBUG = DEBUG

sys.dont_write_bytecode = True

ALLOWED_HOSTS = ['*']

ADMINS = (
    ('Guillaume Pellerin', 'yomguy@parisson.com'),
)

MANAGERS = ADMINS

# Full filesystem path to the project.
#PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
PROJECT_ROOT = '/srv/app/'

DATABASES = {
    'default': {
        'ENGINE': env('ENGINE'),  # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'USER': env('MYSQL_USER'),      # Not used with sqlite3.
        'PASSWORD': env('MYSQL_PASSWORD'),  # Not used with sqlite3.
        'NAME': env('MYSQL_DATABASE'),
        'HOST': 'db',      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '3306',      # Set to empty string for default. Not used with sqlite3.
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/Paris'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
#LANGUAGE_CODE = 'fr_FR'

LANGUAGES = [ ('fr', 'French'),
              ('en', 'English'),
              ('de', 'German'),
              ('zh_CN', 'Simplified Chinese'),
              ('ar_TN', 'Arabic'),
              ('pt_BR', 'Portuguese'),
]

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
# MEDIA_ROOT = PROJECT_ROOT + '/media/'
#
# if not os.path.exists(MEDIA_ROOT):
#     os.makedirs(MEDIA_ROOT)
MEDIA_ROOT = '/srv/media/'

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
# STATIC_ROOT = '/var/www/static'
STATIC_ROOT = '/srv/static/'

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
# Put strings here, like "/home/html/static" or "C:/www/django/static".
# Always use forward slashes, even on Windows.
# Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
'django.contrib.staticfiles.finders.FileSystemFinder',
'django.contrib.staticfiles.finders.AppDirectoriesFinder',
'djangobower.finders.BowerFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)
# Make this unique, and don't share it with anybody.
SECRET_KEY = 'a8l7%06wr2k+3=%#*#@#rvop2mmzko)44%7k(zx%lls^ihm9^5'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)


MIDDLEWARE_CLASSES = (
    # Manage Django URLs for AngularJS with django-angular
    'djng.middleware.AngularUrlMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    # 'pagination.middleware.PaginationMiddleware',
)

ROOT_URLCONF = 'sandbox.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'suit',
    'django.contrib.admin',
    'django.contrib.staticfiles',
    'django_extensions',
    'telemeta',
    'timeside.player',
    'timeside.server',
    'jsonrpc',
    'south',
    'sorl.thumbnail',
    'timezones',
    'jqchat',
    'ipauth',
    'extra_views',
    'debug_toolbar',
    'bootstrap3',
    'bootstrap_pagination',
    'googletools',
    'registration',
    'rest_framework',
    'djcelery',
    'haystack',
    'djangobower',
    'djng',
    'saved_searches',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.request',
    'django.contrib.auth.context_processors.auth',
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    'django.core.context_processors.static',
    'django.contrib.messages.context_processors.messages',
)

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
    'ipauth.backend.RangeBackend',
)

SESSION_ENGINE = "django.contrib.sessions.backends.signed_cookies"

TELEMETA_ORGANIZATION = 'Telemeta'
TELEMETA_SUBJECTS = ('Telemeta', 'web', 'platform', 'audio', 'semantics')
TELEMETA_DESCRIPTION = "Open web audio platform with semantics"
TELEMETA_LOGO = STATIC_URL + 'telemeta/images/logo_telemeta_2.png'

TELEMETA_GMAP_KEY = 'ABQIAAAArg7eSfnfTkBRma8glnGrlxRVbMrhnNNvToCbZQtWdaMbZTA_3RRGObu5PDoiBImgalVnnLU2yN4RMA'

TELEMETA_CACHE_DIR = os.path.join(MEDIA_ROOT, 'cache')
TELEMETA_EXPORT_CACHE_DIR = os.path.join(MEDIA_ROOT, 'export')
TELEMETA_DATA_CACHE_DIR = os.path.join(TELEMETA_CACHE_DIR, 'data')
FILE_UPLOAD_TEMP_DIR = os.path.join(MEDIA_ROOT, 'tmp')
FILE_UPLOAD_PERMISSIONS = 0644

TELEMETA_DOWNLOAD_ENABLED = True
TELEMETA_STREAMING_FORMATS = ('mp3', 'ogg')
TELEMETA_DOWNLOAD_FORMATS = ('wav', 'mp3', 'ogg', 'flac')
TELEMETA_PUBLIC_ACCESS_PERIOD = 51

TELEMETA_STRICT_CODE = False
COLLECTION_PUBLISHED_CODE_REGEX = '*'
COLLECTION_UNPUBLISHED_CODE_REGEX = '*'
ITEM_PUBLISHED_CODE_REGEX = COLLECTION_PUBLISHED_CODE_REGEX + ''
ITEM_UNPUBLISHED_CODE_REGEX = COLLECTION_UNPUBLISHED_CODE_REGEX + ''

AUTH_PROFILE_MODULE = 'telemeta.userprofile'
SESSION_EXPIRE_AT_BROWSER_CLOSE = False

LOGIN_URL = '/login/'
LOGIN_REDIRECT_URL = '/desk/lists/'

EMAIL_HOST = 'localhost'
DEFAULT_FROM_EMAIL = 'webmaster@parisson.com'

TIMESIDE_DEFAULT_GRAPHER_ID = 'waveform_centroid'
TIMESIDE_DEFAULT_WAVEFORM_SIZES = ['346x130', '640x130']
TIMESIDE_AUTO_ZOOM = False

# Settings for django-bootstrap3
BOOTSTRAP3 = {
    'set_required': True,
    'set_placeholder': False,
    'error_css_class': 'has-error',
    'required_css_class': 'has-warning',
    'javascript_in_head': True,
}

PAGINATION_SETTINGS = {
    'PAGE_RANGE_DISPLAYED': 10,
    'MARGIN_PAGES_DISPLAYED': 2,
}

DEBUG_TOOLBAR_PATCH_SETTINGS = False
DEBUG_TOOLBAR_PANELS = [
    'debug_toolbar.panels.versions.VersionsPanel',
    'debug_toolbar.panels.timer.TimerPanel',
    'debug_toolbar.panels.settings.SettingsPanel',
    'debug_toolbar.panels.headers.HeadersPanel',
    'debug_toolbar.panels.request.RequestPanel',
    'debug_toolbar.panels.sql.SQLPanel',
    'debug_toolbar.panels.staticfiles.StaticFilesPanel',
    'debug_toolbar.panels.templates.TemplatesPanel',
    'debug_toolbar.panels.cache.CachePanel',
    'debug_toolbar.panels.signals.SignalsPanel',
    'debug_toolbar.panels.logging.LoggingPanel',
    'debug_toolbar.panels.redirects.RedirectsPanel',
]

SUIT_CONFIG = {
    'ADMIN_NAME': 'Telemeta Admin'
}

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

BROKER_URL = env('BROKER_URL')

CELERY_IMPORTS = ("timeside.server.tasks",)
CELERY_RESULT_BACKEND = 'djcelery.backends.database:DatabaseBackend'
CELERY_TASK_SERIALIZER = 'json'
CELERY_RESULT_SERIALIZER = 'json'
CELERY_ACCEPT_CONTENT = ['application/json']

from worker import app

HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.elasticsearch_backend.ElasticsearchSearchEngine',
        'URL': 'http://search:9200/',
        'INDEX_NAME': 'haystack',
        'INLUDE_SPELLING': True,
    },
}
HAYSTACK_SIGNAL_PROCESSOR = 'haystack.signals.RealtimeSignalProcessor'
HAYSTACK_SEARCH_RESULTS_PER_PAGE = 50

BOWER_COMPONENTS_ROOT = '/srv/bower/'
BOWER_PATH = '/usr/local/bin/bower'
BOWER_INSTALLED_APPS = (
    'jquery',
    'jquery-migrate#~1.2.1',
    'underscore',
    'bootstrap',
    'bootstrap-select#1.5.4',
    'font-awesome#4.4.0',
    'angular#1.2.26',
    'angular-bootstrap-select',
    'angular-resource#1.2.26',
    'raphael',
    'soundmanager#V2.97a.20150601',
    'https://github.com/Parisson/loaders.git',
    'https://github.com/Parisson/ui.git',
    'jquery-ui',
    'tablesorter',
    'video.js',
    'sass-bootstrap-glyphicons',
)



from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User


class Command(BaseCommand):
    help = """Create a default admin user if it doesn't exist.
            you SHOULD change the password and the email afterwards!"""

    username = 'admin'
    password = 'admin'
    email = 'root@example.com'

    def handle(self, *args, **options):
        admin = User.objects.filter(username=self.username)
        if not admin:
            user = User(username=self.username)
            user.set_password(self.password)
            user.email = self.email
            user.is_superuser = True
            user.is_staff = True
            user.save()
            print 'User "'+ self.username + '" created'



# -*- coding: utf-8 -*-
# Copyright (C) 2010 Samalyse SARL
# Copyright (C) 2010-2014 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from __future__ import division
from django.utils.translation import ugettext_lazy as _
from telemeta.models.core import *


class Playlist(ModelCore):
    "Item, collection or marker playlist"
    element_type = 'playlist'
    public_id      = CharField(_('public_id'), required=True)
    author         = ForeignKey(User, related_name="playlists", db_column="author")
    title          = CharField(_('title'), required=True)
    description    = TextField(_('description'))

    class Meta(MetaCore):
        db_table = 'playlists'

    def __unicode__(self):
        return self.title


class PlaylistResource(ModelCore):
    "Playlist components"
    RESOURCE_TYPE_CHOICES = (('item', 'item'), ('collection', 'collection'),
                             ('marker', 'marker'), ('fonds', 'fonds'), ('corpus', 'corpus'))
    element_type       = 'playlist_resource'
    public_id          = CharField(_('public_id'), required=True)
    playlist           = ForeignKey('Playlist', related_name="resources", verbose_name=_('playlist'))
    resource_type      = CharField(_('resource_type'), choices=RESOURCE_TYPE_CHOICES, required=True)
    resource_id        = CharField(_('resource_id'), required=True)

    class Meta(MetaCore):
        db_table = 'playlist_resources'





from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User
from django.template.defaultfilters import slugify

import os
import timeside.core
from timeside.server.models import *
from timeside.core.tools.test_samples import generateSamples
from telemeta.models import *
from telemeta.views.epub import *


class Command(BaseCommand):
    help = "Export all collections of a corpus and itself to EPUB3 files in cache"

    def handle(self, *args, **options):
        corpus_id = args[-1]
        corpus = MediaCorpus.objects.get(public_id=corpus_id)
        book = BaseEpubMixin()
        book.write_book(corpus)
        for collection in corpus.children.all():
            book = BaseEpubMixin()
            book.write_book(corpus, collection=collection)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2011 Parisson SARL
#
# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>


import datetime
import mimetypes
import re, os, random

from django import forms
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core import exceptions
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.core.urlresolvers import reverse, reverse_lazy
from django.db import models
from django.db.models import Q, URLField
from django.db.models.fields import FieldDoesNotExist
from django.forms.models import model_to_dict
from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy as _
from telemeta.models.utils import *
from telemeta.models.fields import *
from telemeta.util.kdenlive_session import *
from telemeta.util.unaccent import unaccent_icmp
from xml.dom.minidom import getDOMImplementation


PUBLIC_ACCESS_CHOICES = (('none', _('none')), ('metadata', _('metadata')),
                         ('mixed', _('mixed')), ('full', _('full')))

public_extra_types = {
    '.webm': 'video/webm',
    '.mp4': 'video/mp4',
}

private_extra_types = {
    '.eaf': 'text/xml',  # ELAN Annotation Format
    '.trs':  'text/xml', # Trancriber Annotation Format
    '.svl':  'text/xml',  # Sonic Visualiser layer file
    '.TextGrid': 'text/praat-textgrid',  # Praat TextGrid annotation file
}

for ext,mime_type in public_extra_types.items():
    mimetypes.add_type(mime_type, ext)

for ext,mime_type in private_extra_types.items():
    mimetypes.add_type(mime_type, ext)

app_name = 'telemeta'

strict_code = getattr(settings, 'TELEMETA_STRICT_CODE', False)


class EnhancedQuerySet(models.query.QuerySet):
    """QuerySet with added functionalities such as WeakForeignKey handling"""

    def delete(self):
        CHUNK=1024
        objects = self.model._meta.get_all_related_objects()
        ii = self.count()
        values = self.values_list('pk')
        for related in objects:
            i = 0
            while i < ii:
                ids = [v[0] for v in values[i:i + CHUNK]]
                filter = {related.field.name + '__pk__in': ids}
                q = related.model.objects.filter(**filter)
                if isinstance(related.field, WeakForeignKey):
                    update = {related.field.name: None}
                    q.update(**update)
                else:
                    q.delete()

                i += CHUNK

        super(EnhancedQuerySet, self).delete()


class EnhancedManager(models.Manager):
    """Manager which is bound to EnhancedQuerySet"""
    def get_query_set(self):
        return EnhancedQuerySet(self.model)


class EnhancedModel(models.Model):
    """Base model class with added functionality. See EnhancedQuerySet"""

    objects = EnhancedManager()

    def delete(self):
        if not self.pk:
            raise Exception("Can't delete without a primary key")
        self.__class__.objects.filter(pk=self.pk).delete()

    class Meta:
        abstract = True


class ModelCore(EnhancedModel):

    @classmethod
    def required_fields(cls):
        required = []
        for field in cls._meta.fields:
            if not field.blank:
                required.append(field)
        return required

    def save(self, force_insert=False, force_update=False, *args, **kwargs):
        required = self.required_fields()
        for field in required:
            if not getattr(self, field.name):
                raise RequiredFieldError(self, field)
        super(ModelCore, self).save(force_insert, force_update, *args, **kwargs)

    @classmethod
    def get_dom_name(cls):
        "Convert the class name to a DOM element name"
        clsname = cls.__name__
        return clsname[0].lower() + clsname[1:]

    @staticmethod
    def get_dom_field_name(field_name):
        "Convert the class name to a DOM element name"
        tokens = field_name.split('_')
        name = tokens[0]
        for t in tokens[1:]:
            name += t[0].upper() + t[1:]
        return name

    def to_dom(self):
        "Return the DOM representation of this media object"
        impl = getDOMImplementation()
        root = self.get_dom_name()
        doc = impl.createDocument(None, root, None)
        top = doc.documentElement
        top.setAttribute("id", str(self.pk))
        fields = self.to_dict()
        for name, value in fields.iteritems():
            element = doc.createElement(self.get_dom_field_name(name))
            if isinstance(value, EnhancedModel):
                element.setAttribute('key', str(value.pk))
            value = unicode(value)
            element.appendChild(doc.createTextNode(value))
            top.appendChild(element)
        return doc

    def to_dict(self):
        "Return model fields as a dict of name/value pairs"
        fields_dict = {}
        for field in self._meta.fields:
            fields_dict[field.name] = getattr(self, field.name)
        return fields_dict

    def to_list(self):
        "Return model fields as a list"
        fields_list = []
        for field in self._meta.fields:
            fields_list.append({'name': field.name, 'value': unicode(getattr(self, field.name))})
        return fields_list

    @classmethod
    def field_label(cls, field_name=None):
        if field_name:
            try:
                return cls._meta.get_field(field_name).verbose_name
            except FieldDoesNotExist:
                try:
                    return getattr(cls, field_name).verbose_name
                except AttributeError:
                    return field_name
        else:
            return cls._meta.verbose_name

    class Meta:
        abstract = True


class MetaCore:
    app_label = 'telemeta'



class CoreQuerySet(EnhancedQuerySet):
    "Base class for all query sets"

    def none(self): # redundant with none() in recent Django svn
        "Return an empty result set"
        return self.extra(where = ["0 = 1"])

    def word_search(self, field, pattern):
        return self.filter(word_search_q(field, pattern))

    def _by_change_time(self, type, from_time = None, until_time = None):
        "Search between two revision dates"
        table = self.model._meta.db_table
        where = []
        if from_time:
            where.append("revisions.time >= '%s'" % from_time.strftime('%Y-%m-%d %H:%M:%S'))
        if until_time:
            where.append("revisions.time <= '%s'" % until_time.strftime('%Y-%m-%d %H:%M:%S'))

        qs = self
        if where:
            where.extend(["revisions.element_type = '%s'" % type, "revisions.element_id = %s.id" % table])
            qs = qs.extra(where = [" AND ".join(where)],
                            tables = ['revisions']).distinct()
        return qs


class CoreManager(EnhancedManager):
    "Base class for all models managers"

    def none(self, *args, **kwargs):
        ""
        return self.get_query_set().none(*args, **kwargs)

    def get(self, **kwargs):
        if kwargs.has_key('public_id'):
            try:
                args = kwargs.copy()
                args['code'] = kwargs['public_id']
                args.pop('public_id')
                return super(CoreManager, self).get(**args)
            except ObjectDoesNotExist:
                args = kwargs.copy()
                args['id'] = kwargs['public_id']
                args.pop('public_id')
                return super(CoreManager, self).get(**args)

        return super(CoreManager, self).get(**kwargs)



from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.core.files.base import ContentFile
from telemeta.models import *
from telemeta.util.unaccent import unaccent
import os


class Command(BaseCommand):
    help = "apply a equal time shift to all markers of an item putting the first one to 0s"
    args = "code"

    def handle(self, *args, **options):
        code = args[-1]

        items = MediaItem.objects.filter(code__contains=code)
        if items:
            item = items[0]
            markers = item.markers.all()
            time_shift = markers[0].time
            for marker in markers:
                marker.time -= time_shift
                marker.save()
        else:
            exit("No such item")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Guillaume Pellerin
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://svn.parisson.org/telemeta/TelemetaLicense.
#
# Author: Guillaume Pellerin <yomguy@parisson.com>
#

import os
import re
import sys
import logging
import datetime
import timeside
from django.core.management import setup_environ
from django.core.files.base import ContentFile

mapping = {
             'title': 'title',
             'album': 'collection',
             'date': 'recorded_from_date',
             'artist': 'author',
             'track-number': 'track',
             'encoder': 'comment',
             'genre': 'generic_style',
             'audio-codec': 'comment',
             'container-format': 'comment',
             }

class Logger:

    def __init__(self, file):
        self.logger = logging.getLogger('myapp')
        self.hdlr = logging.FileHandler(file)
        self.formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        self.hdlr.setFormatter(self.formatter)
        self.logger.addHandler(self.hdlr)
        self.logger.setLevel(logging.INFO)

    def write_info(self, prefix, message):
        self.logger.info(' ' + prefix + ' : ' + message.decode('utf8'))

    def write_error(self, prefix, message):
        self.logger.error(prefix + ' : ' + message.decode('utf8'))


class TelemetaMediaImport:

    def __init__(self, media_dir, log_file):
        self.logger = Logger(log_file)
        self.media_dir = media_dir
        self.medias = self.get_medias()
    
    def get_medias(self):
        os.chdir(self.media_dir)
        file_list = []
        for root, dirs, files in os.walk('.'):
            for file in files:
                path = root + os.sep + file
                if not os.sep+'.' in path:
                    file_list.append({'root': root, 'file': file})
        return file_list
        
    def set_collection(self, collection_name):
        if not collection_name:
            collection_name = 'Unkown'
        code = collection_name.replace(' ','_')
        code = re.sub(r'\W+', '_', code)
        from telemeta.models.media import MediaCollection
        collections = MediaCollection.objects.filter(code=code)
        if not collections:
            collection = MediaCollection(code=code,title=collection_name)
            collection.save()
            msg = 'created'
            self.logger.write_info('collection ' + collection_name, msg)
        else:
            collection = collections[0]
        return collection
        
    def media_import(self):
        from telemeta.models.media import MediaItem
        for media in self.medias:
            path = media['root'] + os.sep + media['file']
            print 'checking ' + path
            filename,  ext = os.path.splitext(media['file'])
            item = MediaItem.objects.filter(code=filename)
            if not item:
                print 'importing ' + path
                decoder = timeside.decoder.FileDecoder(path)
                try:
                    metadata = decoder.metadata()
                    print metadata
                    collection = self.set_collection(metadata['album'])
                    item = MediaItem(collection=collection)
                    item.code = re.sub(r'\W+', '_', metadata['title'])
                    for tag in mapping.keys():
                        try:
                            if tag == 'date':
                                date = metadata[tag].split(' ')[1].split('/')
                                metadata[tag] = date[2]+'-'+date[1]+'-'+date[0]    
                            if mapping[tag] == 'comment':
                                item[mapping[tag]] = item[mapping[tag]] + '\n' + metadata[tag]
                            else:
                                item[mapping[tag]] = metadata[tag]
                        except:
                            continue
                    item.file = path
                    item.save()
                    msg = 'added item : ' + path
                    self.logger.write_info(collection.code, msg)
                except:
                    continue
                

def run():
    project_dir = sys.argv[-2]
    log_file = sys.argv[-1]
    sys.path.append(project_dir)
    import settings
    setup_environ(settings)
    media_dir = settings.MEDIA_ROOT
    t = TelemetaMediaImport(media_dir, log_file)
    t.media_import()

if __name__ == '__main__':
    run()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2009-2010 Guillaume Pellerin <yomguy@parisson.com>

# This file is part of TimeSide.

# TimeSide is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.

# TimeSide is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with TimeSide.  If not, see <http://www.gnu.org/licenses/>.

# Author: Guillaume Pellerin <yomguy@parisson.com>

# for python2.5

version = '0.5'


import os
import sys
import time
import shutil
import datetime
import timeside

# soon with python2.6
#from multiprocessing import Process

from django.core.management import setup_environ
from django.core.files.base import ContentFile
import cgi
fs = cgi.FieldStorage()


orig_media_dir = '/mnt/awdiomusic/musicbase'
project_dir = '/mnt/awdio'
log_file = project_dir + '/logs/process.log'
sys.path.append('/home/awdio/apps/telemeta-awdio')


class GrapherScheme:

    def __init__(self):
        self.color = 255
        self.color_scheme = {
            'waveform': [ # Four (R,G,B) tuples for three main color channels for the spectral centroid method
                        (self.color,self.color,self.color)
#                        (0, 0, 0), (0, 0, 0), (0, 0, 0), (0,0,0)
                        ],
            'spectrogram': [
                        (0, 0, 0), (58/4,68/4,65/4), (80/2,100/2,153/2), (90,180,100), (224,224,44), (255,60,30), (255,255,255)
                        ]}

        # Grapher id
        self.id = 'waveform_awdio'

        # Width of the image
        self.width = 1800

        # Height of the image
        self.height = 233

        # Background color
        self.bg_color = None

        # Force computation. By default, the class doesn't overwrite existing image files.
        self.force = False
        
        # Nb of threads
        # FIXME: memory leak for > 1 !
        self.threads = 1

      
class TelemetaPreprocessImport(object):

    def __init__(self, root_dir, dest_dir, log_file):
	from telemeta.cache import TelemetaCache as Cache
	from telemeta.util.logger import Logger
	self.media_item_dir = 'items'
        self.root_dir = root_dir + 'items'
        self.dest_dir = dest_dir
        self.threads = 1
        self.logger = Logger(log_file)
        self.counter = 0
        self.force = 0
        self.cache = Cache(self.dest_dir)

        self.scheme = GrapherScheme()
        self.width = self.scheme.width
        self.height = self.scheme.height
        self.bg_color = self.scheme.bg_color
        self.color_scheme = self.scheme.color_scheme
        self.force = self.scheme.force
        self.threads = self.scheme.threads
        self.logger = Logger(log_file)
        self.counter = 0
        self.collection_name = 'awdio'
        self.collection = self.set_collection(self.collection_name)
        
        self.analyzers = timeside.core.processors(timeside.api.IAnalyzer)
        self.grapher = timeside.grapher.WaveformAwdio(width=self.width, 
                                                         height=self.height, 
                                                         bg_color=self.bg_color, 
                                                         color_scheme=self.color_scheme)
        

    def set_collection(self, collection_name):
        import telemeta.models
        collections = telemeta.models.media.MediaCollection.objects.filter(code=collection_name)
        if not collections:
            c = telemeta.models.media.MediaCollection(code=collection_name)
            c.title = collection_name
            c.save()
            msg = 'added'
            self.logger.logger.info(collection_name, msg)
            collection = c
        else:
            collection = collections[0]
        return collection

    def process(self):
	import telemeta.models
	keys = fs.keys()
	if keys[0] == 'file':
	    filename = fs['file'].value
	    media_orig = orig_media_dir + os.sep + filename
	    media = self.root_dir + os.sep + filename
	    
	    if not os.path.exists(media):
		shutil.copy(media_orig, media)
		os.system('chmod 644 ' + media)
            
            name, ext = os.path.splitext(filename)
            size = str(self.width) + '_' + str(self.height)
            image_name = name + '.' + self.scheme.id + '.' + size + '.png'
            image = self.dest_dir + os.sep + image_name
            xml = name + '.xml'
            
            if not self.cache.exists(image_name) or not self.cache.exists(xml):
                mess = 'Processing ' + media
                self.logger.logger.info(mess)
	    
		print "Content-type: text/plain\n"
		print mess
		decoder  = timeside.decoder.FileDecoder(media)
		pipe = decoder | self.grapher
		analyzers = []
		analyzers_sub = []
		for analyzer in self.analyzers:
		    subpipe = analyzer()
		    analyzers_sub.append(subpipe)
		    pipe = pipe | subpipe
		pipe.run()
		
		mess = 'Rendering ' + image
		self.logger.logger.info(mess)
		self.grapher.render(output=image)
		
		mess = 'Frames / Pixel = ' + str(self.grapher.graph.samples_per_pixel)
		self.logger.logger.info(mess)
		
		for analyzer in analyzers_sub:
		    value = analyzer.result()
		    if analyzer.id() == 'duration':
			value = datetime.timedelta(0,value)
		    analyzers.append({'name':analyzer.name(),
			    'id':analyzer.id(),
			    'unit':analyzer.unit(),
			    'value':str(value)})
		
		self.cache.write_analyzer_xml(analyzers, xml)
		
		item = telemeta.models.media.MediaItem.objects.filter(code=name)
			    
		if not item:
		    item = telemeta.models.media.MediaItem(collection=self.collection, code=name)
		    item.title = name
		    item.file = self.media_item_dir + os.sep + filename
		    item.save()
		    msg = 'added item : ' + filename
		    self.logger.logger.info(self.collection_name, msg)

		pipe = 0
		decoder = 0
		
		print "OK"
		
		#except:
		    #pipe = 0
		    #decoder = 0
		    #mess = 'Could NOT process : ' + media
		    #self.logger.logger.error(mess)
		    #print mess
		    
	    else:
		mess = "Nothing to do with file : " + media
		self.logger.logger.info(mess)
		print "Content-type: text/plain\n"
		print mess
	
	else:
	    print "Content-type: text/plain\n"
	    print "No file given !"
	

if __name__ == '__main__':
    sys.path.append(project_dir)
    import settings
    setup_environ(settings)
    media_dir = settings.MEDIA_ROOT
    data_dir = settings.TELEMETA_DATA_CACHE_DIR
    t = TelemetaPreprocessImport(media_dir, data_dir, log_file)
    t.process()



# Telemeta OAI-PMH DataSource
#
# Copyright (C) 2009 Samalyse SARL
# Author: Olivier Guilyardi <olivier samalyse com>
#
# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from telemeta.models import MediaCollection, MediaItem, Revision, dublincore
from telemeta.interop.oai import BadArgumentError
from datetime import datetime

class TelemetaOAIDataSource(object):
    """Telemeta OAI datasource adapter. This class implements the oai.IDataSource interface."""

    def get_earliest_time(self):
        """Return the change time of the oldest record(s) as a datetime object"""
        try:
            rev = Revision.objects.order_by('time')[0]
            return rev.time
        except IndexError:
            return datetime.now()

    def prepare_record(self, record):
        ctime = record.get_revision().time
        return dublincore.express_resource(record).to_list(), ctime

    def get_record(self, id):
        """Return a specific record"""
        try:
            record = dublincore.lookup_resource(id)
        except dublincore.MalformedMediaIdentifier, e:
            raise BadArgumentError(e.message)
        return record and self.prepare_record(record)

    def count_records(self, from_time = None, until_time = None):
        """Must return the number of records between (optional) from and until change time."""
        nitems = MediaItem.objects.by_change_time(from_time, until_time).count()
        ncolls = MediaCollection.objects.by_change_time(from_time, until_time).count()
        return nitems + ncolls

    def list_records(self, offset, limit, from_time = None, until_time = None):
        """Return a list of records"""

        result = []

        query = MediaItem.objects.by_change_time(from_time, until_time)
        nitems = query.count()
        if (offset < nitems):
            set = query[offset:offset + limit]
            for record in set:
                result.append(self.prepare_record(record))
            limit -= len(set)
            offset = 0
        else:
            offset -= nitems

        if limit > 0:
            query = MediaCollection.objects.by_change_time(from_time, until_time)
            set = query[offset:offset + limit]
            for record in set:
                result.append(self.prepare_record(record))
            
        return result



# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *


class ProfileView(object):
    """Provide Collections web UI methods"""

    @method_decorator(login_required)
    def profile_detail(self, request, username, template='telemeta/profile_detail.html'):
        user = User.objects.get(username=username)
        try:
            profile = user.get_profile()
        except:
            profile = None
        playlists = get_playlists(request, user)
        user_revisions = get_revisions(25, user)

        return render(request, template, {'profile' : profile, 'usr': user, 'playlists': playlists,
                                          'user_revisions': user_revisions})

    @method_decorator(login_required)
    def profile_edit(self, request, username, template='telemeta/profile_edit.html'):
        if request.user.is_superuser:
            user_hidden_fields = ['profile-user', 'user-password', 'user-last_login', 'user-date_joined']
        else:
            user_hidden_fields = ['user-username', 'user-is_staff', 'profile-user', 'user-is_active',
                         'user-password', 'user-last_login', 'user-date_joined', 'user-groups',
                         'user-user_permissions', 'user-is_superuser', 'profile-expiration_date']

        user = User.objects.get(username=username)
        if user != request.user and not request.user.is_staff:
            mess = ugettext('Access not allowed')
            title = ugettext('User profile') + ' : ' + username + ' : ' + mess
            description = ugettext('Please login or contact the website administator to get a private access.')
            messages.error(request, title)
            return render(request, 'telemeta/messages.html', {'description' : description})

        try:
            profile = user.get_profile()
        except:
            profile = UserProfile(user=user)

        if request.method == 'POST':
            user_form = UserChangeForm(request.POST, instance=user, prefix='user')
            profile_form = UserProfileForm(request.POST, instance=profile, prefix='profile')
            if user_form.is_valid() and profile_form.is_valid():
                user_form.save()
                profile_form.save()
                return redirect('telemeta-desk-profile', username)
        else:
            user_form = UserChangeForm(instance=user, prefix='user')
            profile_form = UserProfileForm(instance=profile, prefix='profile')
            forms = [user_form, profile_form]
        return render(request, template, {'forms': forms, 'usr': user,
                                'user_hidden_fields': user_hidden_fields})




from django.conf import settings

def check():
    """Perform general configuration verifications"""
    if not len(settings.ADMINS):
        raise ConfigurationError("The ADMINS configuration option must be set in settings.py.")

class ConfigurationError(Exception):
    pass



# -*- coding: utf-8 -*-
from telemeta.views.core import *

class SearchViewPublished(ListView):
    """Perform a search through resources"""

    template_name='telemeta/search_results.html'
    paginate_by = 20

    def get_queryset(self):
        input = self.request.GET
        self.criteria = {}

        self.type = None
        if 'type' in self.kwargs:
            self.type = self.kwargs['type']

        self.collections = MediaCollection.objects.enriched()
        self.items = MediaItem.objects.enriched().filter(code__contains="_E_")
        self.corpus = MediaCorpus.objects.all()
        self.fonds  = MediaFonds.objects.all()

        switch = {
            'pattern': lambda value: (
                self.collections.quick_search(value),
                self.items.quick_search(value),
                self.corpus.quick_search(value),
                self.fonds.quick_search(value),
                ),
            'title': lambda value: (
                self.collections.word_search('title', value),
                self.items.by_title(value),
                self.corpus.word_search('title', value),
                self.fonds.word_search('title', value)),
            'location': lambda value: (
                self.collections.by_location(Location.objects.get(name=value)),
                self.items.by_location(Location.objects.get(name=value))),
            'continent': lambda value: (
                self.collections.by_continent(value),
                self.items.filter(continent = value)),
            'ethnic_group': lambda value: (
                self.collections.by_ethnic_group(value),
                self.items.filter(ethnic_group = value),
                EthnicGroup.objects.get(pk=value)),
            'creator': lambda value: (
                self.collections.word_search('creator', value),
                self.items.word_search('collection__creator', value)),
            'collector': lambda value: (
                self.collections.by_fuzzy_collector(value),
                self.items.by_fuzzy_collector(value)),
            'rec_year_from': lambda value: (
                self.collections.by_recording_year(int(value), int(input.get('rec_year_to', value))),
                self.items.by_recording_date(datetime.date(int(value), 1, 1),
                                        datetime.date(int(input.get('rec_year_to', value)), 12, 31))),
            'rec_year_to': lambda value: (self.collections, self.items),
            'pub_year_from': lambda value: (
                self.collections.by_publish_year(int(value), int(input.get('pub_year_to', value))),
                self.items.by_publish_year(int(value), int(input.get('pub_year_to', value)))),
            'pub_year_to': lambda value: (self.collections, self.items),
            'sound': lambda value: (
                self.collections.sound(),
                self.items.sound()),
            'instrument': lambda value: (
                self.collections.by_instrument(value),
                self.items.by_instrument(value)),
        }

        for key, value in input.items():
            func = switch.get(key)
            if func and value and value != "0":
                try:
                    res = func(value)
                    if len(res)  > 4:
                        self.collections, self.items, self.corpus, self.fonds, value = res
                    elif len(res) == 4:
                        self.collections, self.items, self.corpus, self.fonds = res
                    elif len(res) == 3:
                        self.collections, self.items, value = res
                        self.corpus = self.corpus.none()
                        self.fonds = self.fonds.none()
                    else:
                        self.collections, self.items = res
                        self.corpus = self.corpus.none()
                        self.fonds = self.fonds.none()

                except ObjectDoesNotExist:
                    self.collections = self.collections.none()
                    self.items = self.items.none()
                    self.corpus = self.corpus.none()
                    self.fonds = self.fonds.none()

                self.criteria[key] = value

        # Save the search
        user = self.request.user
        if user:
            if user.is_authenticated():
                search = Search(username=user)
                search.save()
                if self.criteria:
                    for key in self.criteria.keys():
                        value = self.criteria[key]
                        if key == 'ethnic_group':
                            try:
                                group = EthnicGroup.objects.get(value=value)
                                value = group.id
                            except:
                                value = ''
                        criter = Criteria(key=key, value=value)
                        criter.save()
                        search.criteria.add(criter)
                    search.save()

        if self.type is None:
            if self.collections.count():
                self.type = 'collections'
            else:
                self.type = 'items'

        if self.type == 'items':
            objects = self.items
        elif self.type == 'collections':
            objects = self.collections
        elif self.type == 'corpus':
            objects = self.corpus
        elif self.type == 'fonds':
            objects = self.fonds

        self.objects = objects
        return objects

    def get_context_data(self, *args, **kwargs):
        context = super(SearchView, self).get_context_data(*args, **kwargs)
        context['criteria'] = self.criteria
        context['collections_num'] =  self.collections.count()
        context['items_num'] = self.items.count()
        context['corpus_num']  = self.corpus.count()
        context['fonds_num'] = self.fonds.count()
        context['type'] = self.type
        context['count'] = self.object_list.count()
        return context


# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *

class AdminView(object):
    """Provide Admin web UI methods"""

    @method_decorator(permission_required('is_superuser'))
    def admin_index(self, request):
        return render(request, 'telemeta/admin.html', self.__get_admin_context_vars())

    @method_decorator(permission_required('is_superuser'))
    def admin_general(self, request):
        return render(request, 'telemeta/admin_general.html', self.__get_admin_context_vars())

    @method_decorator(permission_required('is_superuser'))
    def admin_enumerations(self, request):
        return render(request, 'telemeta/admin_enumerations.html', self.__get_admin_context_vars())

    @method_decorator(permission_required('is_superuser'))
    def admin_users(self, request):
        users = User.objects.all()
        return render(request, 'telemeta/admin_users.html', {'users': users})

    def __get_enumerations_list(self):
        from django.db.models import get_models
        models = get_models(telemeta.models)

        enumerations = []
        for model in models:
            if issubclass(model, Enumeration):
                if not model.hidden:
                    enumerations.append({"name": model._meta.verbose_name,
                                         "id": model._meta.module_name})

        cmp = lambda obj1, obj2: unaccent_icmp(obj1['name'], obj2['name'])
        enumerations.sort(cmp)
        return enumerations

    def __get_admin_context_vars(self):
        return {"enumerations": self.__get_enumerations_list()}

    def __get_enumeration(self, id):
        from django.db.models import get_models
        models = get_models(telemeta.models)
        for model in models:
            if model._meta.module_name == id:
                break

        if model._meta.module_name != id:
            return None

        return model

    @method_decorator(permission_required('telemeta.change_keyword'))
    def edit_enumeration(self, request, enumeration_id):

        enumeration  = self.__get_enumeration(enumeration_id)
        if enumeration == None:
            raise Http404

        vars = self.__get_admin_context_vars()
        vars["enumeration_id"] = enumeration._meta.module_name
        vars["enumeration_name"] = enumeration._meta.verbose_name
        vars["enumeration_values"] = enumeration.objects.all()
        return render(request, 'telemeta/enumeration_edit.html', vars)

    @method_decorator(permission_required('telemeta.add_keyword'))
    def add_to_enumeration(self, request, enumeration_id):

        enumeration  = self.__get_enumeration(enumeration_id)
        if enumeration == None:
            raise Http404

        enumeration_value = enumeration(value=request.POST['value'],
                                        notes=request.POST["notes"])
        enumeration_value.save()

        return self.edit_enumeration(request, enumeration_id)

    @method_decorator(permission_required('telemeta.change_keyword'))
    def update_enumeration(self, request, enumeration_id):

        enumeration  = self.__get_enumeration(enumeration_id)
        if enumeration == None:
            raise Http404

        if request.method == 'POST':
            enumeration.objects.filter(id__in=request.POST.getlist('sel')).delete()

        return self.edit_enumeration(request, enumeration_id)

    @method_decorator(permission_required('telemeta.change_keyword'))
    def edit_enumeration_value(self, request, enumeration_id, value_id):

        enumeration  = self.__get_enumeration(enumeration_id)
        if enumeration == None:
            raise Http404

        record = enumeration.objects.get(id__exact=value_id)
        content_type = ContentType.objects.get(app_label="telemeta", model=enumeration_id)

        vars = self.__get_admin_context_vars()
        vars["enumeration_id"] = enumeration._meta.module_name
        vars["enumeration_name"] = enumeration._meta.verbose_name
        vars["enumeration_record"] = record
        vars["enumeration_records"] = enumeration.objects.all()
        vars['room'] = get_room(content_type=content_type,
                                   id=record.id,
                                   name=record.value)
        return render(request, 'telemeta/enumeration_edit_value.html', vars)

    @method_decorator(permission_required('telemeta.change_keyword'))
    def update_enumeration_value(self, request, enumeration_id, value_id):

        if request.method == 'POST':
            enumeration  = self.__get_enumeration(enumeration_id)
            if enumeration == None:
                raise Http404

            record = enumeration.objects.get(id__exact=value_id)
            record.value = request.POST["value"]
            record.notes = request.POST["notes"]
            record.save()

        return self.edit_enumeration(request, enumeration_id)

    @method_decorator(permission_required('telemeta.change_keyword'))
    def replace_enumeration_value(self, request, enumeration_id, value_id):
        if request.method == 'POST':
            enumeration = self.__get_enumeration(enumeration_id)
            to_value_id = request.POST["value"]
            delete = False
            if 'delete' in request.POST.keys():
                delete = True

        if enumeration == None:
            raise Http404

        from_record = enumeration.objects.get(id__exact=value_id)
        to_record = enumeration.objects.get(id__exact=to_value_id)
        links = [rel.get_accessor_name() for rel in from_record._meta.get_all_related_objects()]
        field_type = WeakForeignKey

        for link in links:
            objects = getattr(from_record, link).all()
            for obj in objects:
                for name in obj._meta.get_all_field_names():
                    try:
                        field = obj._meta.get_field(name)
                        if type(field) == field_type:
                            if field.rel.to == enumeration:
                                setattr(obj, name, to_record)
                                obj.save()
                    except:
                        continue
        if delete:
            from_record.delete()

        return self.edit_enumeration(request, enumeration_id)




from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User
from django.template.defaultfilters import slugify

import os
import timeside.core
from timeside.server.models import *
from timeside.core.tools.test_samples import generateSamples
from telemeta.models import *


class Command(BaseCommand):
    help = "Setup and run a boilerplate for testing"
    cleanup =  True
    code = 'Tests'

    def processor_cleanup(self):
        for processor in Processor.objects.all():
            processor.delete()

    def result_cleanup(self):
        for result in Result.objects.all():
            result.delete()

    def handle(self, *args, **options):
        collection, c = MediaCollection.objects.get_or_create(title=self.code,
                            code=self.code)
        selection, c = Selection.objects.get_or_create(title='Tests')

        if c:
            presets = []
            blacklist =['decoder', 'live', 'gain', 'vamp']
            processors = timeside.core.processor.processors(timeside.core.api.IProcessor)
            for proc in processors:
                trig = True
                for black in blacklist:
                    if black in proc.id():
                        trig = False
                if trig:
                    processor, c = Processor.objects.get_or_create(pid=proc.id())
                    preset, c = Preset.objects.get_or_create(processor=processor, parameters='{}')
                    presets.append(preset)

            media_dir = 'items' + os.sep + 'tests'
            samples_dir = settings.MEDIA_ROOT + media_dir
            samples = generateSamples(samples_dir=samples_dir)

            for sample in samples.iteritems():
                filename, path = sample
                title = os.path.splitext(filename)[0]
                path = media_dir + os.sep + filename
                item, c = Item.objects.get_or_create(title=title, file=path)
                if not item in selection.items.all():
                    selection.items.add(item)
                if self.cleanup:
                    for result in item.results.all():
                        result.delete()
                mediaitem, c = MediaItem.objects.get_or_create(title=title,
                                    code=self.code + '-' + slugify(filename),
                                    file=path, collection=collection)


            experience, c = Experience.objects.get_or_create(title='All')
            for preset in presets:
                if not preset in experience.presets.all():
                    experience.presets.add(preset)

            task = Task(experience=experience, selection=selection)
            task.save()



# -*- coding: utf-8 -*-

"""
Telemeta

U{http://telemeta.org}

@author: Guillaume Pellerin <pellerin@parisson.com>
@author: Paul Brossier <piem@piem.org>
@author: Thomas Fillon <thomas@parisson.com>
@author: Riccardo Zaccarelli <riccardo.zaccarelli@gmail.com>
@author: Olivier Guilyardi <olivier@samalyse.com>
@author: David Lipszyc <davidlipszyc@gmail.com>
@author: Thomas Fillon <thomas.fillon@parisson.com>
"""

__docformat__ = 'epytext en'
__version__ = '1.6'
__url__ = 'http://telemeta.org'
__copyright__ = '(C) 2006-2016 Parisson SARL'
__license__ = 'AGPLv3'
__license_long__ = """
Copyright (C) Parisson SARL (2006-2016)

This file is part of Telemeta.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""



# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>

from django.contrib.auth.models import User
import unittest
from telemeta.models import LocationType, Location, EthnicGroup, Publisher
from telemeta.models import MediaCollection, MediaItem
from datetime import datetime, timedelta
import pytest

pytestmark = pytest.mark.django_db


@pytest.mark.django_db
class CollectionItemTestCase(unittest.TestCase):
    pytestmark = pytest.mark.django_db

    def setUp(self):
        "Create a test database based on objects created in Django"

#        self.david   = User.objects.create(username="david", level="user")
#        self.olivier = User.objects.create(username="olivier", level="admin")
        self.david   = User.objects.create_user(username="david")
        self.olivier = User.objects.create_user(username="olivier")

        self.country = LocationType.objects.create(code="country", name="Country")
        self.continent = LocationType.objects.create(code="continent", name="Continent")
        self.city = LocationType.objects.create(code="city", name="City")

        self.paris = Location.objects.create(name="Paris", type=Location.OTHER_TYPE, complete_type=self.city)
        self.france = Location.objects.create(name="France", type=Location.COUNTRY, complete_type=self.country)
        self.europe = Location.objects.create(name="Europe", type=Location.CONTINENT, complete_type=self.continent)
        self.belgique = Location.objects.create(name="Belgique", type=Location.COUNTRY, complete_type=self.country)

        self.europe.add_child(self.france)
        self.france.add_child(self.paris)
        self.europe.add_child(self.belgique)

        self.a = EthnicGroup.objects.create(value="a")
        self.b = EthnicGroup.objects.create(value="b")
        self.c = EthnicGroup.objects.create(value="c")
        self.d = EthnicGroup.objects.create(value="d")

        self.persepolis = MediaCollection(id=1, code="CNRSMH_E_1970_001_002", reference="A1", title="persepolis",
            creator="Abraham LINCOLN", collector="Friedrich HEINZ", year_published=2009, is_published=True,
            recorded_from_year=1970, recorded_to_year=1980)

        self.persepolis.set_revision(self.david)

        self.volonte = MediaCollection(id=2, reference="A2",  code="CNRSMH_I_1960_001", title="Volonté de puissance",
            creator="Friedrich NIETZSCHE", collector="Jean AMORA", year_published=1999,
            recorded_from_year=1960, recorded_to_year=2000)

        self.volonte.set_revision(self.olivier)

        self.nicolas = MediaCollection(id=3, reference="A3",  code="CNRSMH_I_1967_123", title="petit nicolas",
            creator="Georgette McKenic", collector="Paul MAILLE",  year_published=1999,
            recorded_from_year=1967, recorded_to_year=1968)

        self.nicolas.set_revision(self.olivier)

        self.item_1 = MediaItem(id=1, collection=self.persepolis, code="CNRSMH_E_1970_001_002_44",
            recorded_from_date="1971-01-12", recorded_to_date="1971-02-24", location=self.paris,
            ethnic_group=self.a, title="item 1", author="Mickael SHEPHERD", collector="Charles PREMIER",
            comment="comment 1")

        self.item_1.set_revision(self.david)

        self.item_2 = MediaItem(id=2, collection=self.volonte, code="CNRSMH_I_1960_001_129",
            recorded_from_date="1981-01-12", recorded_to_date="1991-02-24", location=self.france,
            ethnic_group=self.a, title="item 2", author="Rick ROLL", comment="comment 2")

        self.item_2.set_revision(self.david)

        self.item_3 = MediaItem(id=3, collection=self.nicolas, code="CNRSMH_I_1967_123_456_01",
            recorded_from_date="1968-01-12", recorded_to_date="1968-02-24", location=self.belgique,
            ethnic_group=self.b, title="item 3", author="John SMITH", collector="Paul CARLOS",
            comment="comment 3",  )

        self.item_3.set_revision(self.olivier)

        self.item_4 = MediaItem(id=4, collection=self.persepolis, code="CNRSMH_E_1970_001_002_22_33",
            recorded_from_date="1972-01-12", recorded_to_date="1972-02-24", location=self.europe,
            ethnic_group=self.a, title="item 4", alt_title="I4", author="Keanu REAVES",
            collector="Christina BARCELONA", comment="comment 4")

        self.item_4.set_revision(self.olivier)

        self.item_5 = MediaItem(id=5, collection=self.volonte,code="CNRSMH_I_1960_001_789_85_22",
            approx_duration="00:05:00", recorded_from_date="1978-01-12", recorded_to_date="1978-02-24",
            location=self.belgique, ethnic_group=self.a, title="item 5", alt_title="I5",
            author="Simon PAUL", collector="Javier BARDEM",
            comment="comment 5")

        self.item_5.set_revision(self.olivier)

        self.item_6 = MediaItem(id=6, collection=self.persepolis, code="CNRSMH_E_1970_001_002_90",
            recorded_from_date="1968-01-12", recorded_to_date="1968-02-11", location=self.france,
            ethnic_group=self.b, title="item 6", author="Paul ANDERSON",
            collector="Jim CARLSON", comment="comment 10000")

        self.item_6.set_revision(self.david)

        self.collections = MediaCollection.objects.all()
        self.items       = MediaItem.objects.all()

    def tearDown(self):
        User.objects.all().delete()
        LocationType.objects.all().delete()
        Location.objects.all().delete()
        EthnicGroup.objects.all().delete()
        MediaCollection.objects.all().delete()
        MediaItem.objects.all().delete()

    def testQuickSearchOnCollections(self):
        "Test quick_search property of MediaCollection class"
        result = self.collections.quick_search("persepolis")
        self.assertEquals(len(result), 1)
        self.assertEquals(result[0], self.persepolis)
        self.assertEquals(self.collections.quick_search("nietzsche")[0], self.volonte)
        result = self.collections.quick_search("nicolas")
        self.assertEquals(result[0], self.nicolas)

    def testQuickSearchOnItems(self):
        "Test quick_search property of MediaItem class"
        result = self.items.quick_search("item").order_by("title")
        self.assertEquals(result[0], self.item_1)
        self.assertEquals(result[1], self.item_2)
        self.assertEquals(result[2], self.item_3)
        self.assertEquals(result[3], self.item_4)
        self.assertEquals(result[4], self.item_5)
        self.assertEquals(result[5], self.item_6)

    def testWordSearch(self):
        "Test quick_search property of MediaCollection class, specificly quick_search on collection title"
        result = self.collections.quick_search("volonté puissance")
        self.assertEquals(result[0], self.volonte)
        result = self.collections.quick_search("puissance volonté")
        self.assertEquals(result[0], self.volonte)
        result = self.collections.quick_search("volonte puissance")
        self.assertEquals(result[0], self.volonte)
        result = self.collections.quick_search("puissance volonte")
        self.assertEquals(result[0], self.volonte)

    def testLocationSearch(self):
        "Test by_country and by_continent properties of MediaCollection class"
        self.assertEquals(self.collections.by_location(self.france)[0], self.persepolis)
        self.assertEquals(self.collections.by_location(self.europe)[0], self.persepolis)
        self.assertEquals(self.collections.by_location(self.belgique).order_by("title")[0], self.nicolas)
        self.assertEquals(self.collections.by_location(self.belgique).order_by("title")[1], self.volonte)

    def testRecordingYear(self):
        "Test by_recording_year property of MediaCollection class"
        self.assertEquals(self.collections.by_recording_year(1970, 1980)[0], self.persepolis)
        result = self.collections.by_recording_year(1975).order_by("title")
        self.assertEquals(result[0], self.persepolis)
        self.assertEquals(result[1], self.volonte)

    def testPublishYearOnCollection(self):
        "Test by_publish_year property of MediaCollection class"
        result=self.collections.by_publish_year(1999).order_by("title")
        self.assertEquals(result[0], self.nicolas)
        self.assertEquals(result[1], self.volonte)

    def testEthnicGroup(self):
        "Test by_ethnic_group property of MediaCollection class"
        result=self.collections.by_ethnic_group(self.a).order_by("title")
        self.assertEquals(result[0], self.persepolis)
        self.assertEquals(result[1], self.volonte)

    def testRecordingDate(self):
        "Test by_recording_date property of MediaItem class"
        result = self.items.by_recording_date("1968-01-01", "1972-12-12").order_by("title")
        self.assertEquals(result[0], self.item_1)
        self.assertEquals(result[1], self.item_3)
        self.assertEquals(result[2], self.item_4)
        self.assertEquals(result[3], self.item_6)
        result = self.items.by_recording_date("1968-02-06").order_by("title")
        self.assertEquals(result[0], self.item_3)
        self.assertEquals(result[1], self.item_6)

    def testTitle(self):
        "Test by_title property of MediaItem class"
        result = self.items.by_title("item").order_by("title")
        self.assertEquals(result[0], self.item_1)
        self.assertEquals(result[1], self.item_2)
        self.assertEquals(result[2], self.item_3)
        self.assertEquals(result[3], self.item_4)
        self.assertEquals(result[4], self.item_5)
        self.assertEquals(result[5], self.item_6)
        self.item_2.title = ''
        self.item_2.save()
        self.item_5.title = ''
        self.item_5.save()
        result = self.items.by_title("volonté").order_by("title")
        self.assertEquals(result[0], self.item_2)
        self.assertEquals(result[1], self.item_5)
        result = self.items.by_title("puissance volonté").order_by("title")
        self.assertEquals(result[0], self.item_2)
        self.assertEquals(result[1], self.item_5)

    def testPublishYearOnItem(self):
        "Test by_publish_year property of MediaItem class"
        result = self.items.by_publish_year(1999).order_by("title")
        self.assertEquals(result[0], self.item_2)
        self.assertEquals(result[1], self.item_3)
        self.assertEquals(result[2], self.item_5)

    def testWordSearchCore(self):
        "Test word_search property of CoreQuerySet class"
        self.assertEquals(self.collections.word_search("title", "volonté")[0], self.volonte)
        self.assertEquals(self.collections.word_search("code", "CNRSMH_E_1970_001_002")[0], self.persepolis)
        self.assertEquals(self.items.word_search("code", "CNRSMH_E_1970_001_002_44")[0], self.item_1)
        result = self.items.word_search("comment", "comment").order_by("title")
        self.assertEquals(result[0], self.item_1)
        self.assertEquals(result[1], self.item_2)
        self.assertEquals(result[2], self.item_3)
        self.assertEquals(result[3], self.item_4)
        self.assertEquals(result[4], self.item_5)
        self.assertEquals(result[5], self.item_6)

    def testByChangeTimeOnCollection(self):
        "Test by_change_time property of MediaCollection class"
        now = datetime.now()
        print self.collections.all()
        print MediaCollection.objects.all()
        result = self.collections.by_change_time(now - timedelta(hours=1), now).order_by("title")
        self.assertEquals(result[0], self.persepolis)

    def testByChangeTimeOnItem(self):
        "Test by_change_time property of MediaItem class"
        now = datetime.now()
        result = self.items.by_change_time(now - timedelta(hours=1), now).order_by("title")
        self.assertEquals(result[0], self.item_1)

    def testWithoutCollection(self):
        "Test without_collection property of MediaItem class"
        self.assertEquals(self.items.without_collection().count(), 0)

    def testCodeRequired(self):
        "Test that a proper failure occur when a collection code isn't provided"
        c = MediaCollection()
        try:
            c.set_revision(self.olivier)
        except RequiredFieldError, e:
            self.assertEquals(e.field.name, 'code')
        else:
            self.fail("No exception raised")

    def testDomForeignKey(self):
        "Test DOM foreign key embedding"
        doc = self.item_4.to_dom()
        self.assertEquals(doc.getElementsByTagName('collection')[0].getAttribute('key'), str(self.persepolis.id))

    def testLocationRelation(self):
        "Test location country and continent resolving"
        self.assertEquals(self.france, self.item_1.location.countries()[0])
        self.assertEquals(self.europe, self.item_1.location.continents()[0])
        self.assertEquals(self.france, self.item_2.location.countries()[0])
        self.assertEquals(self.europe, self.item_2.location.continents()[0])

    def testCollectionCountries(self):
        "Test the MediaCollection.countries() method"
        self.assertEquals(self.volonte.countries(), [self.belgique, self.france])

pytestmark = pytest.mark.django_db

@pytest.mark.django_db
class RelatedDeleteTestCase(unittest.TestCase):
    pytestmark = pytest.mark.django_db

    def setUp(self):
        self.publisher1 = Publisher.objects.create(id=1, value='publisher1')
        self.publisher2 = Publisher.objects.create(id=2, value='publisher2')
        self.pubcollection1 = PublisherCollection.objects.create(publisher=self.publisher1, value='pub1_collection1')

        self.rights1 = LegalRight.objects.create(id=1, value='right1')

        MediaCollection.objects.create(id=1, code='CNRSMH_I_1256_456', title='Collection1',
                                       publisher=self.publisher1, publisher_collection=self.pubcollection1,
                                       legal_rights=self.rights1)
        MediaCollection.objects.create(id=2, code='CNRSMH_I_1256_123', title='Collection2',
                                       publisher=self.publisher2)

    def tearDown(self):
        Publisher.objects.all().delete()
        PublisherCollection.objects.all().delete()
        LegalRight.objects.all().delete()
        MediaCollection.objects.all().delete()

    def testOnDeleteSetNull(self):
        self.rights1.delete()
        self.assertEquals(LegalRight.objects.filter(id=1).count(), 0)
        q = MediaCollection.objects.filter(id=1)
        self.assertEquals(q.count(), 1)
        self.assertEquals(q[0].legal_rights, None)

    def testOnDeleteCascade(self):
        self.publisher1.delete()
        self.assertEquals(Publisher.objects.filter(id=1).count(), 0)
        self.assertEquals(Publisher.objects.filter(id=2).count(), 1)
        self.assertEquals(PublisherCollection.objects.filter(id=1).count(), 0)

        q = MediaCollection.objects.filter(id=1)
        self.assertEquals(q.count(), 1)
        self.assertEquals(q[0].publisher, None)
        self.assertEquals(q[0].publisher_collection, None)

        q = MediaCollection.objects.filter(id=2)
        self.assertEquals(q.count(), 1)
        self.assertEquals(q[0].publisher, self.publisher2)
        self.assertEquals(q[0].publisher_collection, None)

    def testOnDeleteMultiple(self):
        Publisher.objects.all().delete()
        self.assertEquals(Publisher.objects.count(), 0)
        self.assertEquals(PublisherCollection.objects.count(), 0)
        self.assertEquals(MediaCollection.objects.count(), 2)






# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *


class GeoView(object):
    """Provide Geo web UI methods"""

    def list_continents(self, request):
        continents = MediaItem.objects.all().countries(group_by_continent=True)
        return render(request, 'telemeta/geo_continents.html',
                    {'continents': continents, 'gmap_key': settings.TELEMETA_GMAP_KEY })

    def country_info(self, request, id):
        country = Location.objects.get(pk=id)
        return render(request, 'telemeta/country_info.html', {
            'country': country, 'continent': country.continents()[0]})

    def list_countries(self, request, continent):
        continent = Location.objects.by_flatname(continent)[0]
        countries = MediaItem.objects.by_location(continent).countries()

        return render(request, 'telemeta/geo_countries.html', {
            'continent': continent,
            'countries': countries
        })

    def list_country_collections(self, request, continent, country):
        continent = Location.objects.by_flatname(continent)[0]
        country = Location.objects.by_flatname(country)[0]
        objects = MediaCollection.objects.enriched().by_location(country)
        return list_detail.object_list(request, objects,
            template_name='telemeta/geo_country_collections.html', paginate_by=20,
            extra_context={'country': country, 'continent': continent})

    def list_country_items(self, request, continent, country):
        continent = Location.objects.by_flatname(continent)[0]
        country = Location.objects.by_flatname(country)[0]
        objects = MediaItem.objects.enriched().by_location(country)
        return list_detail.object_list(request, objects,
            template_name='telemeta/geo_country_items.html', paginate_by=20,
            extra_context={'country': country, 'continent': continent})


class GeoCountryCollectionView(ListView):

    model = MediaCollection
    template_name = 'telemeta/geo_country_collections.html'
    paginate_by = 20

    def get_queryset(self):
        country = self.kwargs['country']
        continent = self.kwargs['continent']
        self.continent = Location.objects.by_flatname(continent)[0]
        self.country = Location.objects.by_flatname(country)[0]
        return MediaCollection.objects.enriched().by_location(self.country)

    def get_context_data(self, *args, **kwargs):
        context = super(GeoCountryCollectionView, self).get_context_data(*args, **kwargs)
        context['country'] = self.country
        context['continent'] =  self.continent
        context['count'] = self.object_list.count()
        return context


class GeoCountryItemView(ListView):

    model = MediaItem
    template_name = 'telemeta/geo_country_collections.html'
    paginate_by = 20

    def get_queryset(self):
        country = self.kwargs['country']
        continent = self.kwargs['continent']
        self.continent = Location.objects.by_flatname(continent)[0]
        self.country = Location.objects.by_flatname(country)[0]
        return MediaItem.objects.enriched().by_location(self.country)

    def get_context_data(self, *args, **kwargs):
        context = super(GeoCountryItemView, self).get_context_data(*args, **kwargs)
        context['country'] = self.country
        context['continent'] =  self.continent
        context['count'] = self.object_list.count()
        return context





from __future__ import absolute_import
import os, sys
from celery import Celery
from django.conf import settings

sys.path.append(os.path.dirname('sandbox'))

# set the default Django settings module for the 'celery' program.
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'settings')

app = Celery('sandbox')

# Using a string here means the worker will not have to
# pickle the object when using Windows.
app.config_from_object('django.conf:settings')
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)
# app.conf.update(
#     CELERY_RESULT_BACKEND='djcelery.backends.database:DatabaseBackend',
# )

@app.task(bind=True)
def debug_task(self):
    print('Request: {0!r}'.format(self.request))



# -*- coding: utf-8 -*-

import os, re, urllib


class URLMediaParser(object):

    formats = ['mp3', 'wav', 'ogg', 'flac', 'aac', 'mp4', 'webm']

    def __init__(self, url):
        self.url = url
        self.formats = self.get_formats()
        self.urls = []
        if self.url[-1] != '/':
        	self.url += '/'

    def get_formats(self):
    	formats = []
    	for f in self.formats:
    		formats.append(f.upper())
    		formats.append(f.lower())
    		formats.append(f.capitalize())
    	return formats

    def get_urls(self):    
        data = urllib.urlopen(self.url).read()
        for line in data.split("\012"):
            s = re.compile('href=".*\.*"').search(line,1)
            if s:
                filename = line[s.start():s.end()].split('"')[1]
                name, ext = os.path.splitext(filename)
                if ext[1:] in self.formats:
                    self.urls.append(self.url + filename)
        return self.urls


if __name__ == "__main__":
	import sys
	parser = URLMediaParser(sys.argv[-1])
	urls = parser.get_urls()
	print urls
	print len(urls)


# -*- coding: utf-8 -*-
""" xmltodict(): convert xml into tree of Python dicts.

This was copied and modified from John Bair's recipe at aspn.activestate.com:
	http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/149368
"""
import os
import string
import locale
from xml.parsers import expat

# If we're in Dabo, get the default encoding.
#import dabo
#import dabo.lib.DesignerUtils as desUtil
#from dabo.dLocalize import _
#from dabo.lib.utils import resolvePath
#app = dabo.dAppRef
#if app is not None:
	#default_encoding = app.Encoding
#else:
		#enc = locale.getlocale()[1]
		#if enc is None:
			#enc = dabo.defaultEncoding
		#default_encoding = enc

# Python seems to need to compile code with \n linesep:
code_linesep = "\n"
eol = os.linesep
default_encoding = 'utf-8'
default_decoding = 'utf-8'

class Xml2Obj:
	"""XML to Object"""
	def __init__(self):
		self.root = None
		self.nodeStack = []
		self.attsToSkip = []
		self._inCode = False
		self._mthdName = ""
		self._mthdCode = ""
		self._codeDict = None
		self._inProp = False
		self._propName = ""
		self._propData = ""
		self._propDict = None
		self._currPropAtt = ""
		self._currPropDict = None


	def StartElement(self, name, attributes):
		"""SAX start element even handler"""
		if name == "code":
			# This is code for the parent element
			self._inCode = True
			parent = self.nodeStack[-1]
			if not parent.has_key("code"):
				parent["code"] = {}
				self._codeDict = parent["code"]

		elif name == "properties":
			# These are the custom property definitions
			self._inProp = True
			self._propName = ""
			self._propData = ""
			parent = self.nodeStack[-1]
			if not parent.has_key("properties"):
				parent["properties"] = {}
				self._propDict = parent["properties"]

		else:
			if self._inCode:
				self._mthdName = name.encode()
			elif self._inProp:
				if self._propName:
					# In the middle of a prop definition
					self._currPropAtt = name.encode()
				else:
					self._propName = name.encode()
					self._currPropDict = {}
					self._currPropAtt = ""
			else:
				element = {"name": name.encode()}
				if len(attributes) > 0:
					for att in self.attsToSkip:
						if attributes.has_key(att):
							del attributes[att]
					element["attributes"] = attributes

				# Push element onto the stack and make it a child of parent
				if len(self.nodeStack) > 0:
					parent = self.nodeStack[-1]
					if not parent.has_key("children"):
						parent["children"] = []
					parent["children"].append(element)
				else:
					self.root = element
				self.nodeStack.append(element)


	def EndElement(self, name):
		"""SAX end element event handler"""
		if self._inCode:
			if name == "code":
				self._inCode = False
				self._codeDict = None
			else:
				# End of an individual method
				mth = self._mthdCode.strip()
				if not mth.endswith("\n"):
					mth += "\n"
				self._codeDict[self._mthdName] = mth
				self._mthdName = ""
				self._mthdCode = ""
		elif self._inProp:
			if name == "properties":
				self._inProp = False
				self._propDict = None
			elif name == self._propName:
				# End of an individual prop definition
				self._propDict[self._propName] = self._currPropDict
				self._propName = ""
			else:
				# end of a property attribute
				self._currPropDict[self._currPropAtt] = self._propData
				self._propData = self._currPropAtt = ""
		else:
			self.nodeStack = self.nodeStack[:-1]


	def CharacterData(self, data):
		"""SAX character data event handler"""
		if self._inCode or data.strip():
			data = data.replace("&lt;", "<")
			data = data.encode(default_encoding)
			if self._inCode:
				if self._mthdCode:
					self._mthdCode += data
				else:
					self._mthdCode = data
			elif self._inProp:
				self._propData += data
			else:
				element = self.nodeStack[-1]
				if not element.has_key("cdata"):
					element["cdata"] = ""
				element["cdata"] += data


	def Parse(self, xml):
		# Create a SAX parser
		Parser = expat.ParserCreate(default_encoding)
		# SAX event handlers
		Parser.StartElementHandler = self.StartElement
		Parser.EndElementHandler = self.EndElement
		Parser.CharacterDataHandler = self.CharacterData
		# Parse the XML File
		ParserStatus = Parser.Parse(xml, 1)
		return self.root


	def ParseFromFile(self, filename):
		return self.Parse(open(filename,"r").read())


def xmltodict(xml, attsToSkip=[], addCodeFile=False):
	"""Given an xml string or file, return a Python dictionary."""
	parser = Xml2Obj()
	parser.attsToSkip = attsToSkip
	isPath = os.path.exists(xml)
	errmsg = ""
	if eol not in xml and isPath:
		# argument was a file
		try:
			ret = parser.ParseFromFile(xml)
		except expat.ExpatError, e:
			errmsg = "The XML in '%s' is not well-formed and cannot be parsed: %s" % (xml, e)
	else:
		# argument must have been raw xml:
		if not xml.strip().startswith("<?xml "):
			# it's a bad file name
			errmsg = "The file '%s' could not be found" % xml
		else:
			try:
				ret = parser.Parse(xml)
			except expat.ExpatError:
				errmsg = "An invalid XML string was encountered"
	if errmsg:
		raise Exception(errmsg)

	if addCodeFile and isPath:
		# Get the associated code file, if any
		codePth = "%s-code.py" % os.path.splitext(xml)[0]
		if os.path.exists(codePth):
			try:
				codeDict = desUtil.parseCodeFile(open(codePth).read())
				desUtil.addCodeToClassDict(ret, codeDict)
			except StandardError, e:
				print "Failed to parse code file:", e
	return ret


def escQuote(val, noEscape=False, noQuote=False):
	"""Add surrounding quotes to the string, and escape
	any illegal XML characters.
	"""
	if not isinstance(val, basestring):
		val = str(val)
	if not isinstance(val, unicode):
		val = unicode(val, default_encoding)
	if noQuote:
		qt = ''
	else:
		qt = '"'
	slsh = "\\"
# 	val = val.replace(slsh, slsh+slsh)
	if not noEscape:
		# First escape internal ampersands. We need to double them up due to a
		# quirk in wxPython and the way it displays this character.
		val = val.replace("&", "&amp;&amp;")
		# Escape any internal quotes
		val = val.replace('"', '&quot;').replace("'", "&apos;")
		# Escape any high-order characters
		chars = []
		for pos, char in enumerate(list(val)):
			if ord(char) > 127:
				chars.append("&#%s;" % ord(char))
			else:
					chars.append(char)
		val = "".join(chars)
	val = val.replace("<", "&#060;").replace(">", "&#062;")
	return "%s%s%s" % (qt, val, qt)


def dicttoxml(dct, level=0, header=None, linesep=None):
	"""Given a Python dictionary, return an xml string.

	The dictionary must be in the format returned by dicttoxml(), with keys
	on "attributes", "code", "cdata", "name", and "children".

	Send your own XML header, otherwise a default one will be used.

	The linesep argument is a dictionary, with keys on levels, allowing the
	developer to add extra whitespace depending on the level.
	"""
	att = ""
	ret = ""

	if dct.has_key("attributes"):
		for key, val in dct["attributes"].items():
			# Some keys are already handled.
			noEscape = key in ("sizerInfo",)
			val = escQuote(val, noEscape)
			att += " %s=%s" % (key, val)
	ret += "%s<%s%s" % ("\t" * level, dct["name"], att)

	if (not dct.has_key("cdata") and not dct.has_key("children")
			and not dct.has_key("code") and not dct.has_key("properties")):
		ret += " />%s" % eol
	else:
		ret += ">"
		if dct.has_key("cdata"):
			ret += "%s" % dct["cdata"].decode(default_encoding).replace("<", "&lt;")

		if dct.has_key("code"):
			if len(dct["code"].keys()):
				ret += "%s%s<code>%s" % (eol, "\t" * (level+1), eol)
				methodTab = "\t" * (level+2)
				for mthd, cd in dct["code"].items():
					# Convert \n's in the code to eol:
					cd = eol.join(cd.splitlines())

					# Make sure that the code ends with a linefeed
					if not cd.endswith(eol):
						cd += eol

					ret += "%s<%s><![CDATA[%s%s]]>%s%s</%s>%s" % (methodTab,
							mthd, eol, cd, eol,
							methodTab, mthd, eol)
				ret += "%s</code>%s"	% ("\t" * (level+1), eol)

		if dct.has_key("properties"):
			if len(dct["properties"].keys()):
				ret += "%s%s<properties>%s" % (eol, "\t" * (level+1), eol)
				currTab = "\t" * (level+2)
				for prop, val in dct["properties"].items():
					ret += "%s<%s>%s" % (currTab, prop, eol)
					for propItm, itmVal in val.items():
						itmTab = "\t" * (level+3)
						ret += "%s<%s>%s</%s>%s" % (itmTab, propItm, itmVal,
								propItm, eol)
					ret += "%s</%s>%s" % (currTab, prop, eol)
				ret += "%s</properties>%s"	% ("\t" * (level+1), eol)

		if dct.has_key("children") and len(dct["children"]) > 0:
			ret += eol
			for child in dct["children"]:
				ret += dicttoxml(child, level+1, linesep=linesep)
		indnt = ""
		if ret.endswith(eol):
			# Indent the closing tag
			indnt = ("\t" * level)
		ret += "%s</%s>%s" % (indnt, dct["name"], eol)

		if linesep:
			ret += linesep.get(level, "")

	if level == 0:
		if header is None:
			header = '<?xml version="1.0" encoding="%s" standalone="no"?>%s' \
					% (default_encoding, eol)
		ret = header + ret

	return ret


def flattenClassDict(cd, retDict=None):
	"""Given a dict containing a series of nested objects such as would
	be created by restoring from a cdxml file, returns a dict with all classIDs
	as keys, and a dict as the corresponding value. The dict value will have
	keys for the attributes and/or code, depending on what was in the original
	dict. The end result is to take a nested dict structure and return a flattened
	dict with all objects at the top level.
	"""
	if retDict is None:
		retDict = {}
	atts = cd.get("attributes", {})
	props = cd.get("properties", {})
	kids = cd.get("children", [])
	code = cd.get("code", {})
	classID = atts.get("classID", "")
	classFile = resolvePath(atts.get("designerClass", ""))
	superclass = resolvePath(atts.get("superclass", ""))
	superclassID = atts.get("superclassID", "")
	if superclassID and os.path.exists(superclass):
		# Get the superclass info
		superCD = xmltodict(superclass, addCodeFile=True)
		flattenClassDict(superCD, retDict)
	if classID:
		if os.path.exists(classFile):
			# Get the class info
			classCD = xmltodict(classFile, addCodeFile=True)
			classAtts = classCD.get("attributes", {})
			classProps = classCD.get("properties", {})
			classCode = classCD.get("code", {})
			classKids = classCD.get("children", [])
			currDict = retDict.get(classID, {})
			retDict[classID] = {"attributes": classAtts, "code": classCode,
					"properties": classProps}
			retDict[classID].update(currDict)
			# Now update the child objects in the dict
			for kid in classKids:
				flattenClassDict(kid, retDict)
		else:
			# Not a file; most likely just a component in another class
			currDict = retDict.get(classID, {})
			retDict[classID] = {"attributes": atts, "code": code,
					"properties": props}
			retDict[classID].update(currDict)
	if kids:
		for kid in kids:
			flattenClassDict(kid, retDict)
	return retDict


def addInheritedInfo(src, super, updateCode=False):
	"""Called recursively on the class container structure, modifying
	the attributes to incorporate superclass information. When the
	'updateCode' parameter is True, superclass code is added to the
	object's code
	"""
	atts = src.get("attributes", {})
	props = src.get("properties", {})
	kids = src.get("children", [])
	code = src.get("code", {})
	classID = atts.get("classID", "")
	if classID:
		superInfo = super.get(classID, {"attributes": {}, "code": {}, "properties": {}})
		src["attributes"] = superInfo["attributes"].copy()
		src["attributes"].update(atts)
		src["properties"] = superInfo.get("properties", {}).copy()
		src["properties"].update(props)
		if updateCode:
			src["code"] = superInfo["code"].copy()
			src["code"].update(code)
	if kids:
		for kid in kids:
			addInheritedInfo(kid, super, updateCode)



#if __name__ == "__main__":
	#test_dict = {"name": "test", "attributes":{"path": "c:\\temp\\name",
			#"problemChars": "Welcome to <Jos\xc3\xa9's \ Stuff!>\xc2\xae".decode("latin-1")}}
	#print "test_dict:", test_dict
	#xml = dicttoxml(test_dict)
	#print "xml:", xml
	#test_dict2 = xmltodict(xml)
	#print "test_dict2:", test_dict2
	#print "same?:", test_dict == test_dict2



# -*- coding: utf-8 -*-
#
# Copyright (C) 2007 Samalyse SARL
# Copyright (C) 2003-2005 Edgewall Software
# Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
# Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
# Author: Jonas Borgström <jonas@edgewall.com>
#         Christopher Lenz <cmlenz@gmx.de>
#         Olivier Guilyardi <olivier@samalyse.com>

__all__ = ['TelemetaError']


class TelemetaError(Exception):
    """Exception base class for errors in Telemeta."""
    pass




# -*- coding: utf-8 -*-
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Guillaume Pellerin <yomguy@parisson.com>

from telemeta.views.home import *
from telemeta.views.item import *
from telemeta.views.collection import *
from telemeta.views.admin import *
from telemeta.views.instrument import *
from telemeta.views.geo import *
from telemeta.views.marker import *
from telemeta.views.playlist import *
from telemeta.views.profile import *
from telemeta.views.feed import *
from telemeta.views.resource import *
from telemeta.views.epub import *
from telemeta.views.search import *
from telemeta.views.new_playlist import *



# -*- coding: utf-8 -*-

import logging

class Logger:
    """A logging object"""

    def __init__(self, file):
        self.logger = logging.getLogger('myapp')
        self.hdlr = logging.FileHandler(file)
        self.formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        self.hdlr.setFormatter(self.formatter)
        self.logger.addHandler(self.hdlr)
        self.logger.setLevel(logging.INFO)

    def debug(self, msg):
        self.logger.error('DEBUG : ' + message)
        



# -*- coding: utf-8 -*-
# Copyright (C) 2010 Samalyse SARL
# Copyright (C) 2010-2014 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from __future__ import division
from django.utils.translation import ugettext_lazy as _
from telemeta.models.core import *
from telemeta.models.query import *
from telemeta.models.identifier import *
from telemeta.models.resource import *

# Special code regex of collections for the branch
collection_published_code_regex = getattr(settings, 'COLLECTION_PUBLISHED_CODE_REGEX', '[A-Za-z0-9._-]*')
collection_unpublished_code_regex = getattr(settings, 'COLLECTION_UNPUBLISHED_CODE_REGEX', '[A-Za-z0-9._-]*')

# CREM
#collection_published_code_regex   = 'CNRSMH_E_[0-9]{4}(?:_[0-9]{3}){2}'
#collection_unpublished_code_regex = 'CNRSMH_I_[0-9]{4}_[0-9]{3}'

collection_code_regex = '(?:%s|%s)' % (collection_published_code_regex,
                                       collection_unpublished_code_regex)

class MediaCollection(MediaResource):
    "Describe a collection of items"

    element_type = 'collection'

    def is_valid_collection_code(value):
        "Check if the collection code is well formed"
        regex = '^' + collection_code_regex + '$'
        if not re.match(regex, value):
            raise ValidationError(u'%s is not a valid collection code' % value)

    # General informations
    title                 = CharField(_('title'), required=True)
    alt_title             = CharField(_('original title / translation'))
    creator               = CharField(_('depositor / contributor'), help_text=_('First name, Last name ; First name, Last name'))
    description           = TextField(_('description'))
    recording_context     = WeakForeignKey('RecordingContext', related_name="collections", verbose_name=_('recording context'))
    recorded_from_year    = IntegerField(_('recording year (from)'), help_text=_('YYYY'))
    recorded_to_year      = IntegerField(_('recording year (until)'), help_text=_('YYYY'))
    year_published        = IntegerField(_('year published'), help_text=_('YYYY'))
    public_access         = CharField(_('access type'), choices=PUBLIC_ACCESS_CHOICES, max_length=16, default="metadata")

    # Geographic and cultural informations
    # See "countries" and "ethnic_groups" methods below

    # Legal notices
    collector             = CharField(_('recordist'), help_text=_('First name, Last name ; First name, Last name'))
    publisher             = WeakForeignKey('Publisher', related_name="collections", verbose_name=_('publisher'))
    publisher_collection  = WeakForeignKey('PublisherCollection', related_name="collections", verbose_name=_('publisher collection'))
    publisher_serial      = CharField(_('publisher serial number'))
    booklet_author        = CharField(_('booklet author'), blank=True)
    reference             = CharField(_('publisher reference'))
    external_references   = TextField(_('bibliographic references'))

    auto_period_access    = BooleanField(_('automatic access after a rolling period'), default=True)
    legal_rights          = WeakForeignKey('LegalRight', related_name="collections", verbose_name=_('legal rights'))

    # Archiving data
    code                  = CharField(_('code'), unique=True, required=True, validators=[is_valid_collection_code])
    old_code              = CharField(_('old code'), unique=False, null=True, blank=True)
    acquisition_mode      = WeakForeignKey('AcquisitionMode', related_name="collections", verbose_name=_('mode of acquisition'))
    cnrs_contributor      = CharField(_('CNRS depositor'))
    copy_type             = WeakForeignKey('CopyType', related_name="collections", verbose_name=_('copy type'))
    metadata_author       = WeakForeignKey('MetadataAuthor', related_name="collections", verbose_name=_('record author'))
    booklet_description   = TextField(_('related documentation'))
    publishing_status     = WeakForeignKey('PublishingStatus', related_name="collections", verbose_name=_('secondary edition'))
    status                = WeakForeignKey('Status', related_name="collections", verbose_name=_('collection status'))
    alt_copies            = TextField(_('copies'))
    comment               = TextField(_('comment'))
    metadata_writer       = WeakForeignKey('MetadataWriter', related_name="collections", verbose_name=_('record writer'))
    archiver_notes        = TextField(_('archiver notes'))
    items_done            = CharField(_('items finished'))
    collector_is_creator  = BooleanField(_('recordist identical to depositor'))
    is_published          = BooleanField(_('published'))
    conservation_site     = CharField(_('conservation site'))

    # Technical data
    media_type            = WeakForeignKey('MediaType', related_name="collections", verbose_name=_('media type'))
    approx_duration       = DurationField(_('estimated duration'), help_text='hh:mm:ss')
    physical_items_num    = IntegerField(_('number of components (medium / piece)'))
    original_format       = WeakForeignKey('OriginalFormat', related_name="collections", verbose_name=_('original format'))
    physical_format       = WeakForeignKey('PhysicalFormat', related_name="collections", verbose_name=_('archive format'))
    ad_conversion         = WeakForeignKey('AdConversion', related_name='collections', verbose_name=_('digitization'))

    # No more used old fields
    alt_ids               = CharField(_('copies (obsolete field)'))
    travail               = CharField(_('archiver notes (obsolete field)'))

    # All
    objects               = MediaCollectionManager()

    exclude = ['alt_ids', 'travail']

    permissions = (("can_download_collection_epub", "Can download collection EPUB"),)

    class Meta(MetaCore):
        db_table = 'media_collections'
        ordering = ['code']
        verbose_name = _('collection')

    def __unicode__(self):
        return self.code

    def save(self, force_insert=False, force_update=False, *args, **kwargs):
        super(MediaCollection, self).save(force_insert, force_update, *args, **kwargs)

    @property
    def public_id(self):
        return self.code

    @property
    def has_mediafile(self):
        "Tell wether this collection has any media files attached to its items"
        items = self.items.all()
        for item in items:
            if item.file:
                return True
        return False

    def __name_cmp(self, obj1, obj2):
        return unaccent_icmp(obj1.name, obj2.name)

    def countries(self):
        "Return the countries of the items"
        countries = []
        for item in self.items.filter(location__isnull=False):
            for country in item.location.countries():
                if not country in countries:
                    countries.append(country)
        countries.sort(self.__name_cmp)
        return countries
    countries.verbose_name = _("states / nations")

    def main_countries(self):
        "Return the main countries of the items (no aliases or ancestors)"
        countries = []
        for item in self.items.filter(location__isnull=False):
            if not item.location in countries:
                countries.append(item.location)
        countries.sort(self.__name_cmp)
        return countries
    main_countries.verbose_name = _("states / nations")

    def ethnic_groups(self):
        "Return the ethnic groups of the items"
        groups = []
        items = self.items.all()
        for item in items:
            if item.ethnic_group and not item.ethnic_group in groups:
                groups.append(item.ethnic_group)

        cmp = lambda a, b: unaccent_icmp(a.value, b.value)
        groups.sort(cmp)

        return groups
    ethnic_groups.verbose_name = _('populations / social groups')

    def computed_duration(self):
        duration = Duration()
        for item in self.items.all():
            duration += item.computed_duration()
        return duration
    computed_duration.verbose_name = _('computed duration')

    def computed_size(self):
        "Return the total size of a collection"
        size = 0
        for item in self.items.all():
            size += item.size()
        return size
    computed_size.verbose_name = _('collection size')

    def document_status(self):
        if '_I_' in self.public_id:
            return ugettext('Unpublished')
        elif '_E_' in self.public_id:
            return ugettext('Published')
        else:
            return ''

    def get_url(self):
        return get_full_url(reverse('telemeta-collection-detail', kwargs={'public_id':self.pk}))

    def to_dict_with_more(self):
        # metadata = model_to_dict(self, fields=[], exclude=self.exclude)
        metadata = self.to_dict()
        for key in self.exclude:
            if key in metadata.keys():
                del metadata[key]

        metadata['url'] = get_full_url(reverse('telemeta-collection-detail', kwargs={'public_id':self.pk}))
        metadata['doc_status'] = self.document_status()
        metadata['countries'] = ';'.join([location.name for location in self.main_countries()])
        metadata['ethnic_groups'] = ';'.join([group.value for group in self.ethnic_groups()])
        revision = self.get_revision()
        if revision:
            metadata['last_modification_date'] = unicode(revision.time)
        metadata['computed_duration'] = unicode(self.computed_duration())
        metadata['computed_size'] = unicode(self.computed_size())
        metadata['number_of_items'] = unicode(self.items.all().count())
        metadata['approx_duration'] = unicode(self.approx_duration)

        i = 0
        for media in self.related.all():
            metadata['related_media_title' + '_' + str(i)] = media.title
            if media.url:
                tag = 'related_media_url' + '_' + str(i)
                metadata[tag] = media.url
            elif media.url:
                metadata[tag] = get_full_url(reverse('telemeta-collection-related',
                                            kwargs={'public_id': self.public_id, 'media_id': media.id}))
            i += 1

        # One ID only
        identifiers = self.identifiers.all()
        if identifiers:
            identifier = identifiers[0]
            metadata['identifier_id'] = identifier.identifier
            metadata['identifier_type'] = identifier.type
            metadata['identifier_date'] = unicode(identifier.date_last)
            metadata['identifier_note'] = identifier.notes

        # All IDs
        # i = 0
        # for indentifier in self.identifiers.all():
        #     metadata['identifier' + '_' + str(i)] = identifier.identifier
        #     metadata['identifier_type' + '_' + str(i)] = identifier.type
        #     metadata['identifier_date_last' + '_' + str(i)] = unicode(identifier.date_last)
        #     metadata['identifier_notes' + '_' + str(i)] = identifier.notes
        #     i += 1

        return metadata

    def get_json(self):
        import json
        return json.dumps(self.to_dict_with_more())

    def to_row(self, tags):
        row = []
        _dict = self.to_dict_with_more()
        for tag in tags:
            if tag in _dict.keys():
                row.append(_dict[tag])
            else:
                row.append('')
        return row


class MediaCollectionRelated(MediaRelated):
    "Collection related media"

    collection      = ForeignKey('MediaCollection', related_name="related", verbose_name=_('collection'))

    class Meta(MetaCore):
        db_table = 'media_collection_related'
        verbose_name = _('collection related media')
        verbose_name_plural = _('collection related media')


class MediaCollectionIdentifier(Identifier):
    """Collection identifier"""

    collection = ForeignKey(MediaCollection, related_name="identifiers", verbose_name=_('collection'))

    class Meta(MetaCore):
        db_table = 'media_collection_identifier'
        verbose_name = _('collection identifier')
        verbose_name_plural = _('collection identifiers')
        unique_together = ('identifier', 'collection')



# -*- coding: utf-8 -*-
import multiprocessing
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

import sys


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


CLASSIFIERS = ['Environment :: Web Environment',
'Framework :: Django',
'Intended Audience :: Science/Research',
'Intended Audience :: Education',
'Programming Language :: Python',
'Programming Language :: JavaScript',
'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
'Topic :: Multimedia :: Sound/Audio',
'Topic :: Multimedia :: Sound/Audio :: Analysis',
'Topic :: Multimedia :: Sound/Audio :: Players',
'Topic :: Scientific/Engineering :: Information Analysis',
'Topic :: System :: Archiving',  ]


setup(
  name = "Telemeta",
  url = "http://telemeta.org",
  description = "Open web audio application with semantics",
  long_description = open('README.rst').read(),
  author = "Guillaume Pellerin",
  author_email = "yomguy@parisson.com",
  version = '1.6',
  install_requires = [
        'django==1.6.11',
        'django-registration==1.0',
        'django-extensions',
        'django-timezones',
        'django-jqchat',
        'django-debug-toolbar==1.3.2',
        'django-extra-views',
        'django-breadcrumbs',
        'django-bootstrap3',
        'django-bootstrap-pagination',
        'django-json-rpc',
        'django-suit',
        'django-google-tools',
        'django-ipauth',
        'django-celery',
        'timeside>=0.7',
        'south',
        'docutils',
        'psutil',
        'pyyaml',
        'python-ebml',
        'zipstream',
        'elasticsearch==1.6.0',
        'django-haystack',
        'ebooklib',
        'django-environ',
        'redis',
        'Werkzeug',
    ],
  tests_require=['pytest-django', 'pytest-cov', 'factory-boy'],
  # Provide a test command through django-setuptest
  cmdclass={'test': PyTest},
  platforms=['OS Independent'],
  license='CeCILL v2',
  classifiers = CLASSIFIERS,
  packages = find_packages(),
  include_package_data = True,
  zip_safe = False,
)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Guillaume Pellerin
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://svn.parisson.org/telemeta/TelemetaLicense.
#
# Author: Guillaume Pellerin <yomguy@parisson.com>
#

import logging
import codecs
import os
import sys
import csv
import logging
import datetime
from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User
from django.core.files.base import ContentFile
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.template.defaultfilters import slugify

from telemeta.models import *
from telemeta.util.unaccent import unaccent


class Logger:

    def __init__(self, file):
        self.logger = logging.getLogger('myapp')
        self.hdlr = logging.FileHandler(file)
        self.formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        self.hdlr.setFormatter(self.formatter)
        self.logger.addHandler(self.hdlr)
        self.logger.setLevel(logging.INFO)

    def info(self, prefix, message):
        self.logger.info(' ' + prefix + ' : ' + message.decode('utf8'))

    def error(self, prefix, message):
        self.logger.error(prefix + ' : ' + message.decode('utf8'))


class Command(BaseCommand):

    """Import CREM collections from collection directories containing media files
    and eventually a XLS files representing the relation between old codes and new codes
    """

    help = "import CREM collections (special usecase)"
    admin_email = 'webmaster@parisson.com'
    media_root = settings.MEDIA_ROOT

    option_list = BaseCommand.option_list + (
          make_option('-d', '--dry-run',
            action='store_true',
            dest='dry-run',
            help='Do NOT write anything'),
          make_option('-f', '--force',
            action='store_true',
            dest='force',
            help='Force overwrite data'),
          make_option('-s', '--source',
            dest='source_dir',
            help='define the source directory'),
          make_option('-l', '--log',
            dest='log',
            help='define log file'),
          make_option('-p', '--pattern',
            dest='pattern',
            help='define the pattern'),
    )

    def write_file(self, item, media):
        filename = media.split(os.sep)[-1]
        print 'importing ' + filename
        if os.path.exists(media):
            if not item.file or self.force:
                if not self.media_root in self.source_dir:
                    print "file not in MEDIA_ROOT, copying..."
                    f = open(media, 'r')
                    if not self.dry_run:
                        file_content = ContentFile(f.read())
                        item.file.save(filename, file_content)
                    f.close()
                else:
                    print "file in MEDIA_ROOT, linking..."
                    path = media.replace(self.media_root, '')
                    if not self.dry_run:
                        item.file = path
                item.save()
                if self.user:
                    item.set_revision(self.user)
            else:
                msg = item.code + ' : fichier ' + item.file.name + ' deja inscrit dans la base de donnees et pas de forcage !'
                self.logger.info('item', msg)
        else:
            msg = item.code + ' : fichier audio ' + filename + ' inexistant dans le dossier !'
            self.logger.error('item', msg)

    def handle(self, *args, **kwargs):
        self.logger = Logger(kwargs.get('log'))
        self.pattern = kwargs.get('pattern')
        self.source_dir = os.path.abspath(kwargs.get('source_dir'))
        self.dry_run =  kwargs.get('dry-run')
        self.force = kwargs.get('force')

        self.domain = Site.objects.all()[0].domain
        self.user = User.objects.filter(username='admin')[0]
        self.collections = os.listdir(self.source_dir)

        collections = []
        for collection in self.collections:
            collection_dir = self.source_dir + os.sep + collection
            collection_files = os.listdir(collection_dir)

            if not '/.' in collection_dir and self.pattern in collection_dir:
                collection_name = collection.split(os.sep)[-1]
                collections.append(collection_name)
                c = MediaCollection.objects.filter(code=collection_name)

                if not c and collection + '.csv' in collection_files:
                    msg = collection + ' collection NON présente dans la base de données, SORTIE '
                    self.logger.error(collection, msg)
                    sys.exit(msg)
                elif not c:
                    msg = 'collection NON présente dans la base de données, CREATION '
                    self.logger.info(collection, msg)
                    if not self.dry_run:
                        c = MediaCollection(code=collection_name, title=collection_name)
                        c.save()
                        c.set_revision(self.user)
                else:
                    msg = 'collection présente dans la base de données, SELECTION'
                    self.logger.info(collection, msg)

        for collection in collections:
            collection_dir = self.source_dir + os.sep + collection
            collection_name = collection
            collection_files = os.listdir(collection_dir)
            msg = '************************ ' + collection + ' ******************************'
            self.logger.info(collection, msg[:70])
            csv_file = ''
            rows = {}

            if collection + '.csv' in collection_files:
                csv_file = self.source_dir + os.sep + collection + os.sep + collection + '.csv'
                csv_data = csv.reader(open(csv_file), delimiter=';')
                for row in csv_data:
                    rows[row[1].strip()] = row[0].strip()
                msg = collection + ' import du fichier CSV de la collection'
                self.logger.info(collection, msg[:70])
            else:
                msg = collection + ' pas de fichier CSV dans la collection'
                self.logger.info(collection, msg[:70])

            c = MediaCollection.objects.filter(code=collection_name)
            if not c:
                if not self.dry_run:
                    c = MediaCollection(code=collection_name)
                    c.save()
                msg = ' collection NON présente dans la BDD, CREATION '
                self.logger.info(c.code, msg)
            else:
                c = c[0]
                msg = ' id = '+str(c.id)
                self.logger.info(c.code, msg)

            audio_files = []
            for file in collection_files:
                ext = ['WAV', 'wav']
                if file.split('.')[-1] in ext and file[0] != '.':
                    audio_files.append(file)

            audio_files.sort()
            nb_items = c.items.count()
            counter = 0

            for file in audio_files:
                code = file.split('.')[0]
                wav_file = self.source_dir + os.sep + collection + os.sep + file

                if len(audio_files) <= nb_items:
                    items = MediaItem.objects.filter(code=code)

                    old_ref = ''
                    if code in rows and not items:
                        old_ref = rows[code]
                        items = MediaItem.objects.filter(old_code=old_ref)

                    if items:
                        item = items[0]
                        msg = code + ' : ' + item.old_code + ' : Cas 1 ou 2 : id = ' + str(item.id)
                        self.logger.info('item', msg)
                        item.code = code
                    else:
                        item = MediaItem(code=code, collection=c)
                        msg = code + ' : ' + old_ref + ' : Cas 1 ou 2 : item NON présent dans la base de données, CREATION'
                        self.logger.info('item', msg)

                    self.write_file(item, wav_file)

                elif nb_items == 1 and len(audio_files) > 1:
                    if counter == 0:
                        msg = code + ' : Cas 3a : item n°01 présent dans la base de données, PASSE'
                        self.logger.info('item', msg)
                    else:
                        item = MediaItem(code=code, collection=c)
                        msg = code + ' : Cas 3a : item NON présent dans la base de données, CREATION'
                        self.logger.info('item', msg)
                        self.write_file(item, wav_file)

                elif nb_items > 1 and nb_items < len(audio_files):
                    msg = code + ' : Cas 3b : nb items < nb de fichiers audio, PAS de creation'
                    self.logger.info('item', msg)

                counter += 1

        msg = 'Liste des URLs des collections importées :'
        self.logger.info('INFO', msg)
        for collection in collections:
            msg = 'http://'+self.domain+'/archives/collections/'+collection
            self.logger.info(collection, msg)





from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User
from django.template.defaultfilters import slugify
from django.utils import translation
from telemeta.models import *
from telemeta.util.unaccent import unaccent
import logging
import codecs
from xlwt import Workbook

class Command(BaseCommand):
    help = "Export media fields to a XLS file (see an example in example/data/"
    args = "path"
    first_row = 1
    admin_email = 'webmaster@parisson.com'
    language_codes = ['en_US', 'fr_FR', 'de_DE']
    models = [MediaFonds, MediaCorpus, MediaCollection, MediaItem]
    width = 256

    def handle(self, *args, **options):
        self.file = args[0]
        self.book = Workbook()
        for model in self.models:
            self.sheet = self.book.add_sheet(model.element_type)
            self.sheet.write(0, 0, 'Field')
            self.sheet.col(0).width = self.width*32

            k = 1
            for language_code in self.language_codes:
                self.sheet.write(0, k, language_code)
                self.sheet.col(k).width = self.width*32
                k += 1

            i = 1
            for field in model._meta.fields:
                self.sheet.write(i, 0, field.attname)
                j = 1
                for language_code in self.language_codes:
                    translation.activate(language_code)
                    self.sheet.write(i, j, unicode(field.verbose_name.lower()))
                    j += 1
                i += 1

        self.book.save(self.file)



# -*- coding: utf-8 -*-

# Copyright (C) 2015 Angy Fils-Aimé, Killian Mary

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from haystack import indexes
from telemeta.models import *


class MediaItemIndex(indexes.SearchIndex, indexes.Indexable):

    text = indexes.CharField(document=True, use_template=True)
    item_acces = indexes.CharField(model_attr='collection__public_access', faceted=True)
    item_status = indexes.CharField(model_attr='collection__document_status', faceted=True)
    digitized = indexes.BooleanField(default=False, faceted=True)
    media_type = indexes.CharField(model_attr='media_type', null='None', faceted=True)
    recording_context = indexes.CharField(model_attr='collection__recording_context', default='', faceted=True)
    physical_format = indexes.CharField(model_attr='collection__physical_format', default='', faceted=True)
    #content_auto = indexes.EdgeNgramField(model_attr='content')

    #advance search
    title = indexes.NgramField(model_attr='title')
    code = indexes.NgramField(model_attr='code', default='')
    location_principal = indexes.CharField(null='None', boost=1.05)
    location_relation = indexes.CharField()
    ethnic_group = indexes.CharField(model_attr='ethnic_group', default='')
    instruments = indexes.NgramField(default='')
    collectors = indexes.NgramField(model_attr='collector', default='')
    recorded_from_date = indexes.DateField(model_attr='recorded_from_date', null='None')
    recorded_to_date = indexes.DateField(model_attr='recorded_to_date', null='None')
    year_published = indexes.IntegerField(model_attr='collection__year_published', default='')

    def prepare_digitized(self, obj):
        if obj.file.name:
            return True
        elif '/' in obj.url:
            return True
        else:
            return False

    def get_model(self):
        return MediaItem

    def prepare_location_principal(self, obj):
        if obj.location is not None:
            return u"".join(obj.location.name)
        else:
            return None

    def prepare_location_relation(self, obj):
        location = []
        if obj.location is not None:
            location_alias = LocationAlias.objects.filter(location__name=obj.location)
            location_rela = LocationRelation.objects.filter(location__name=obj.location)
            for rela in location_rela:
                location.append(rela.ancestor_location.name)
            for alias in location_alias:
                location.append(alias.alias)
            #print u"".join(' ' + local for local in location).encode("utf-8")
            #print u"%s".encode("utf-8") % location
            #print [local for local in location]
        return u"".join(' ' + local for local in location)

    def prepare_instruments(self, obj):
        item = MediaItemPerformance.objects.all().filter(media_item__exact=obj)
        instruments = []
        for material in item:
            if material.instrument is not None:
                instruments.append(material.instrument.name)
            if material.alias is not None:
                instruments.append(material.alias.name)
        return u"".join(' ' + instru for instru in instruments)

    def prepare_collectors(self, obj):
        collectors = []
        collectors.append(obj.collection.collector)
        collectors.append(obj.collector)
        return u"".join(' ' + collector for collector in collectors)


class MediaCollectionIndex(indexes.SearchIndex, indexes.Indexable):

    text = indexes.CharField(document=True, use_template=True)
    item_acces = indexes.CharField(model_attr='public_access', faceted=True)
    item_status = indexes.CharField(model_attr='document_status', faceted=True)
    digitized = indexes.BooleanField(default=False, faceted=True)
    media_type = indexes.CharField(model_attr='media_type', null='None', faceted=True)
    recording_context = indexes.CharField(model_attr='recording_context', default='', faceted=True)
    physical_format = indexes.CharField(model_attr='physical_format', default='', faceted=True)
    #content_auto = indexes.EdgeNgramField(model_attr='content')

    #advance search
    title = indexes.NgramField(model_attr='title')
    code = indexes.NgramField(model_attr='code', default='')
    location_principal = indexes.CharField(default='', boost=1.05)
    location_relation = indexes.CharField()
    ethnic_group = indexes.CharField(default='')
    instruments = indexes.NgramField(default='')
    collectors = indexes.NgramField(model_attr='collector', default='')
    recorded_from_date = indexes.DateField(model_attr='recorded_from_year', null=True)
    recorded_to_date = indexes.DateField(model_attr='recorded_to_year', null=True)
    year_published = indexes.IntegerField(model_attr='year_published', default='')

    def prepare_digitized(self, obj):
        return obj.has_mediafile

    def get_model(self):
        return MediaCollection

    def prepare_location_principal(self, obj):
        collec_location = []
        for item in obj.items.all():
            location = []
            if item.location is not None:
                collec_location.append(item.location.name)
        return u"".join(' ' + location for location in collec_location)

    def prepare_location_relation(self, obj):
        collec_location = []
        for item in obj.items.all():
            location = []
            if item.location is not None:
                location_alias = LocationAlias.objects.filter(location__name=item.location)
                location_rela = LocationRelation.objects.filter(location__name=item.location)
                for rela in location_rela:
                    location.append(rela.ancestor_location.name)
                for alias in location_alias:
                    location.append(alias.alias)
                for name in location:
                    if name and not name in collec_location:
                        collec_location.append(name)
        return u"".join(' ' + location for location in collec_location)

    def prepare_ethnic_group(self, obj):
        return "%s" % obj.ethnic_groups()

    def prepare_instruments(self, obj):
        instruments = []
        items = obj.items.all()
        for item in items:
            materials = MediaItemPerformance.objects.all().filter(media_item__exact=item)
            for material in materials:
                if material.instrument and not material.instrument in instruments:
                    instruments.append(material.instrument.name)

                if material.alias and not material.alias in instruments:
                    instruments.append(material.alias.name)
        return u"".join(' ' + instru for instru in instruments)

    def prepare_recorded_from_date(self, obj):
        if obj.recorded_from_year != 0:
            return datetime.date(int(obj.recorded_from_year), 01, 01)
        else:
            return None

    def prepare_recorded_to_date(self, obj):
        if obj.recorded_to_year != 0:
            return datetime.date(int(obj.recorded_to_year), 01, 01)
        else:
            return None


class MediaCorpusIndex(indexes.SearchIndex, indexes.Indexable):

    text = indexes.CharField(document=True, use_template=True)
    #item_acces = indexes.CharField(model_attr='public_access', faceted=True)
    #item_status = indexes.CharField(model_attr='document_status', faceted=True)
    digitized = indexes.BooleanField(default=False, faceted=True)
    #media_type = indexes.CharField(model_attr='media_type', null='None', faceted=True)
    #recording_context = indexes.CharField(model_attr='recording_context', default='', faceted=True)
    #physical_format = indexes.CharField(model_attr='collection__physical_format', default='', faceted=True)
    #content_auto = indexes.EdgeNgramField(model_attr='content')

    #advance search
    title = indexes.NgramField(model_attr='title')
    code = indexes.NgramField(model_attr='code', default='')
    #location_principal = indexes.CharField(default='', boost=1.05)
    #location_relation = indexes.CharField()
    #ethnic_group = indexes.CharField(default='')
    #instruments = indexes.NgramField(default='')
    #collectors = indexes.NgramField(model_attr='collector', default='')
    #recorded_from_date = indexes.DateField(model_attr='recorded_from_year', null=True)
    recorded_to_date = indexes.DateField(model_attr='recorded_to_year', null=True)
    #year_published = indexes.IntegerField(model_attr='year_published', default='')

    def prepare_digitized(self, obj):
        return obj.has_mediafile

    def get_model(self):
        return MediaCorpus


class MediaFondsIndex(indexes.SearchIndex, indexes.Indexable):

    text = indexes.CharField(document=True, use_template=True)
    #item_acces = indexes.CharField(model_attr='public_access', faceted=True)
    #item_status = indexes.CharField(model_attr='document_status', faceted=True)
    digitized = indexes.BooleanField(default=False, faceted=True)
    #media_type = indexes.CharField(model_attr='media_type', null='None', faceted=True)
    #recording_context = indexes.CharField(model_attr='recording_context', default='', faceted=True)
    #physical_format = indexes.CharField(model_attr='physical_format', default='', faceted=True)
    #content_auto = indexes.EdgeNgramField(model_attr='content')

    #advance search
    title = indexes.NgramField(model_attr='title')
    code = indexes.NgramField(model_attr='code', default='')
    #location_principal = indexes.CharField(default='', boost=1.05)
    #location_relation = indexes.CharField()
    #ethnic_group = indexes.CharField(default='')
    #instruments = indexes.NgramField(default='')
    #collectors = indexes.NgramField(model_attr='collector', default='')
    #recorded_from_date = indexes.DateField(model_attr='recorded_from_year', null=True)
    #recorded_to_date = indexes.DateField(model_attr='recorded_to_year', null=True)
    #year_published = indexes.IntegerField(model_attr='year_published', default='')

    def prepare_digitized(self, obj):
        return obj.has_mediafile

    def get_model(self):
        return MediaFonds



# -*- coding: utf-8 -*-
# Copyright (C) 2010 Samalyse SARL
# Copyright (C) 2010-2014 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>
#          Guillaume Pellerin <yomguy@parisson.com>

from __future__ import division
from django.utils.translation import ugettext_lazy as _
from telemeta.models.core import *
from telemeta.models.resource import *
from telemeta.models.query import *
from telemeta.models.identifier import *
from telemeta.models.resource import *
from telemeta.models.enum import *


item_published_code_regex = getattr(settings, 'ITEM_PUBLISHED_CODE_REGEX', '[A-Za-z0-9._-]*')
item_unpublished_code_regex = getattr(settings, 'ITEM_UNPUBLISHED_CODE_REGEX', '[A-Za-z0-9._-]*')

item_code_regex = '(?:%s|%s)' % (item_published_code_regex, item_unpublished_code_regex)

ITEM_PUBLIC_ACCESS_CHOICES = (('none', _('none')), ('metadata', _('metadata')),
                         ('full', _('full')))

ITEM_TRANSODING_STATUS = ((0, _('broken')), (1, _('pending')), (2, _('processing')),
                         (3, _('done')), (5, _('ready')))


class MediaItem(MediaResource):
    "Describe an item"

    element_type = 'item'

    # Main Informations
    title                 = CharField(_('title'))
    alt_title             = CharField(_('original title / translation'))
    collector             = CharField(_('collector'), help_text=_('First name, Last name ; First name, Last name'))
    collection            = ForeignKey('MediaCollection', related_name="items", verbose_name=_('collection'))
    recorded_from_date    = DateField(_('recording date (from)'), help_text=_('YYYY-MM-DD'))
    recorded_to_date      = DateField(_('recording date (until)'), help_text=_('YYYY-MM-DD'))
    public_access         = CharField(_('access type'), choices=ITEM_PUBLIC_ACCESS_CHOICES, max_length=16, default="metadata")

    # Geographic and cultural informations
    location              = WeakForeignKey('Location', verbose_name=_('location'))
    location_comment      = CharField(_('location details'))
    cultural_area         = CharField(_('cultural area'))
    language              = CharField(_('language'))
    language_iso          = ForeignKey('Language', related_name="items", verbose_name=_('Language (ISO norm)'), blank=True, null=True, on_delete=models.SET_NULL)
    ethnic_group          = WeakForeignKey('EthnicGroup', related_name="items", verbose_name=_('population / social group'))
    context_comment       = TextField(_('Ethnographic context'))

    # Musical informations
    moda_execut           = CharField(_('implementing rules'))
    vernacular_style      = WeakForeignKey('VernacularStyle', related_name="items", verbose_name=_('vernacular style'))
    generic_style         = WeakForeignKey('GenericStyle', related_name="items", verbose_name=_('generic style'))
    author                = CharField(_('author / compositor'), help_text=_('First name, Last name ; First name, Last name'))

    # Legal mentions
    organization          = WeakForeignKey('Organization', verbose_name=_('organization'))
    depositor             = CharField(_('depositor'))
    rights                = WeakForeignKey('Rights', verbose_name=_('rights'))

    # Archiving data
    code                  = CharField(_('code'), unique=True, blank=True, required=True, help_text=_('CollectionCode_ItemCode'))
    old_code              = CharField(_('original code'), unique=False, blank=True)
    track                 = CharField(_('item number'))
    collector_selection   = CharField(_('collector selection'))
    collector_from_collection = BooleanField(_('collector as in collection'))
    creator_reference     = CharField(_('creator reference'))
    external_references   = TextField(_('published references'))
    auto_period_access    = BooleanField(_('automatic access after a rolling period'), default=True)
    comment               = TextField(_('remarks'))

    # Technical data
    media_type            = WeakForeignKey('MediaType', related_name="items", verbose_name=_('media type'))
    approx_duration       = DurationField(_('approximative duration'), blank=True, help_text=_('hh:mm:ss'))
    mimetype              = CharField(_('mime type'), max_length=255, blank=True)
    file                  = FileField(_('file'), upload_to='items/%Y/%m/%d', db_column="filename", max_length=1024)
    url                   = URLField(_('URL'), max_length=512, blank=True)

    # LAM
    recordist             = CharField(_('recordist'))
    digitalist            = CharField(_('digitalist'))
    digitization_date     = DateField(_('digitization date'))
    publishing_date       = DateField(_('publishing date'))
    scientist             = CharField(_('scientist'), help_text=_('First name, Last name ; First name, Last name'))
    topic                 = WeakForeignKey('Topic', verbose_name=_('topic'))
    summary               = TextField(_('summary'))
    contributor           = CharField(_('contributor'))

    # Manager
    objects               = MediaItemManager()

    exclude = ['copied_from_item', 'mimetype',
                    'organization', 'depositor', 'rights',
                    'recordist', 'digitalist', 'digitization_date',
                    'publishing_date', 'scientist', 'topic',
                    'summary', 'contributor', ]

    restricted = ['copied_from_item', 'mimetype',
                    'organization', 'depositor', 'rights',
                    'recordist', 'digitalist', 'digitization_date',
                    'publishing_date', 'scientist', 'topic',
                    'summary', 'contributor', 'public_access']


    def keywords(self):
        return ContextKeyword.objects.filter(item_relations__item = self)
    keywords.verbose_name = _('keywords')

    @property
    def public_id(self):
        if self.code:
            return self.code
        return str(self.id)

    @property
    def mime_type(self):
        if not self.mimetype:
            if self.file:
                if os.path.exists(self.file.path):
                    self.mimetype = mimetypes.guess_type(self.file.path)[0]
                    self.save()
                    return self.mimetype
                else:
                    return 'none'
            else:
                return 'none'
        else:
            return _('none')


    class Meta(MetaCore):
        db_table = 'media_items'
        permissions = (("can_play_all_items", "Can play all media items"),
                       ("can_download_all_items", "Can download all media items"), )
        verbose_name = _('item')

    def is_valid_code(self, code):
        "Check if the item code is well formed"
        if not re.match('^' + self.collection.code, self.code):
            return False
        if self.collection.is_published:
            regex = '^' + item_published_code_regex + '$'
        else:
            regex = '^' + item_unpublished_code_regex + '$'
        if re.match(regex, code):
            return True
        return False

    def clean(self):
        if strict_code:
            if self.code and not self.is_valid_code(self.code):
                raise ValidationError("%s is not a valid item code for collection %s"
                                            % (self.code, self.collection.code))

    def save(self, force_insert=False, force_update=False, *args, **kwargs):
        super(MediaItem, self).save(force_insert, force_update, *args, **kwargs)

    def computed_duration(self):
        "Tell the length in seconds of this item media data"
        return self.approx_duration

    computed_duration.verbose_name = _('computed duration')

    def __unicode__(self):
        if self.title and not re.match('^ *N *$', self.title):
            title = self.title
        else:
            title = unicode(self.collection.title)
        if self.track:
            title += ' ' + self.track
        return title

    def get_source(self):
        source = None
        if self.file and os.path.exists(self.file.path):
            source = self.file.path
        elif self.url:
            source = self.url
        return source

    @property
    def instruments(self):
        "Return the instruments of the item"
        instruments = []
        performances = MediaItemPerformance.objects.filter(media_item=self)
        for performance in performances:
            instrument = performance.instrument
            alias = performance.alias
            if not instrument in instruments:
                instruments.append(instrument)
            if not alias in instruments:
                instruments.append(alias)
        #no reference for __name_cmp anywhere
        instruments.sort(self.__name_cmp)
        return instruments

        instruments.verbose_name = _("instruments")

    def size(self):
        if self.file and os.path.exists(self.file.path):
            return self.file.size
        else:
            return 0
    size.verbose_name = _('item size')

    def get_url(self):
        return get_full_url(reverse('telemeta-item-detail', kwargs={'public_id':self.pk}))

    def to_dict_with_more(self):
        # metadata = model_to_dict(self, fields=[], exclude=self.exclude)
        metadata = self.to_dict()
        for key in self.exclude:
            if key in metadata.keys():
                del metadata[key]

        metadata['url'] = self.get_url()
        revision = self.get_revision()
        if revision:
            time = unicode(revision.time)
        else:
            time = ''
        metadata['last_modification_date'] = time
        metadata['collection'] = self.collection.get_url()

        keywords = []
        for keyword in self.keywords():
            keywords.append(keyword.value)
        metadata['keywords'] = ';'.join(keywords)

        related_media_urls = []
        for media in self.related.all():
            if media.url:
                related_media_urls.append(media.url)
            else:
                try:
                    url = get_full_url(reverse('telemeta-item-related',
                                                kwargs={'public_id': self.public_id, 'media_id': media.id}))
                except:
                    url = ''
                related_media_urls.append(url)
        metadata['related_media_urls'] = ';'.join(related_media_urls)

        instruments = []
        instrument_vernacular_names = []
        performers = []
        for performance in self.performances.all():
            if performance.instrument:
                instruments.append(performance.instrument.name)
            if performance.alias:
                instrument_vernacular_names.append(performance.alias.name)
            if performance.musicians:
                performers.append(performance.musicians.replace(' et ', ';'))
        metadata['instruments'] = ';'.join(instruments)
        metadata['instrument_vernacular_names'] = ';'.join(instrument_vernacular_names)
        metadata['performers'] = ';'.join(performers)

        analyzers = ['channels', 'samplerate', 'duration', 'resolution', 'mime_type']
        for analyzer_id in analyzers:
            analysis = MediaItemAnalysis.objects.filter(item=self, analyzer_id=analyzer_id)
            if analysis:
                if analyzer_id == 'duration':
                    value = ':'.join([str('%.2d' % int(float(t))) for t in analysis[0].value.split(':')])
                else:
                    value = analysis[0].value
                metadata[analyzer_id] = value
            elif analyzer_id == 'duration':
                metadata[analyzer_id] = self.approx_duration
            else:
                metadata[analyzer_id] = ''

        metadata['file_size'] = unicode(self.size())
        metadata['thumbnail'] = get_full_url(reverse('telemeta-item-visualize',
                                            kwargs={'public_id': self.public_id,
                                                    'grapher_id': 'waveform_centroid',
                                                    'width': 346,
                                                    'height': 130}))
        # One ID only
        identifiers = self.identifiers.all()
        if identifiers:
            identifier = identifiers[0]
            metadata['identifier_id'] = identifier.identifier
            metadata['identifier_type'] = identifier.type
            metadata['identifier_date'] = unicode(identifier.date_last)
            metadata['identifier_note'] = identifier.notes
        else:
            metadata['identifier_id'] = ''
            metadata['identifier_type'] = ''
            metadata['identifier_date'] = ''
            metadata['identifier_note'] = ''

        # Collection
        metadata['recording_context'] = self.collection.recording_context
        metadata['description_collection'] = self.collection.description
        metadata['status'] = self.collection.status
        metadata['original_format'] = self.collection.original_format
        metadata['physical_format'] = self.collection.physical_format
        metadata['year_published'] = self.collection.year_published
        metadata['publisher'] = self.collection.publisher
        metadata['publisher_collection'] = self.collection.publisher_collection
        metadata['reference_collection'] = self.collection.reference

        return metadata

    def to_row(self, tags):
        row = []
        _dict = self.to_dict_with_more()
        for tag in tags:
            if tag in _dict.keys():
                row.append(_dict[tag])
            else:
                row.append('')
        return row

class MediaItemRelated(MediaRelated):
    "Item related media"

    item = ForeignKey('MediaItem', related_name="related", verbose_name=_('item'))

    def parse_markers(self, **kwargs):
        # Parse KDEnLive session
        if self.file:
            if self.is_kdenlive_session():
                session = KDEnLiveSession(self.file.path)
                markers = session.markers(**kwargs)
                for marker in markers:
                    m = MediaItemMarker(item=self.item)
                    m.public_id = get_random_hash()
                    m.time = float(marker['time'])
                    m.title = marker['comment']
                    m.save()
                return markers

    class Meta(MetaCore):
        db_table = 'media_item_related'
        verbose_name = _('item related media')
        verbose_name_plural = _('item related media')


class MediaItemKeyword(ModelCore):
    "Item keyword"
    item    = ForeignKey('MediaItem', verbose_name=_('item'), related_name="keyword_relations")
    keyword = ForeignKey('ContextKeyword', verbose_name=_('keyword'), related_name="item_relations")

    class Meta(MetaCore):
        db_table = 'media_item_keywords'
        unique_together = (('item', 'keyword'),)


class MediaItemPerformance(ModelCore):
    "Item performance"
    media_item      = ForeignKey('MediaItem', related_name="performances", verbose_name=_('item'))
    instrument      = WeakForeignKey('Instrument', related_name="performances", verbose_name=_('composition'))
    alias           = WeakForeignKey('InstrumentAlias', related_name="performances", verbose_name=_('vernacular name'))
    instruments_num = CharField(_('number'))
    musicians       = CharField(_('interprets'))

    class Meta(MetaCore):
        db_table = 'media_item_performances'


class MediaItemAnalysis(ModelCore):
    "Item analysis result computed by TimeSide"

    element_type = 'analysis'
    item  = ForeignKey('MediaItem', related_name="analysis", verbose_name=_('item'))
    analyzer_id = CharField(_('id'), required=True)
    name = CharField(_('name'))
    value = CharField(_('value'))
    unit = CharField(_('unit'))

    class Meta(MetaCore):
        db_table = 'media_analysis'
        ordering = ['name']

    def to_dict(self):
        if self.analyzer_id == 'duration':
            if '.' in self.value:
                value = self.value.split('.')
                self.value = '.'.join([value[0], value[1][:2]])
        return {'id': self.analyzer_id, 'name': self.name, 'value': self.value, 'unit': self.unit}



class MediaItemMarker(MediaResource):
    "2D marker object : text value vs. time (in seconds)"

    element_type = 'marker'

    item            = ForeignKey('MediaItem', related_name="markers", verbose_name=_('item'))
    public_id       = CharField(_('public_id'), required=True)
    time            = FloatField(_('time (s)'))
    title           = CharField(_('title'))
    date            = DateTimeField(_('date'), auto_now=True)
    description     = TextField(_('description'))
    author          = ForeignKey(User, related_name="markers", verbose_name=_('author'),
                                 blank=True, null=True)

    class Meta(MetaCore):
        db_table = 'media_markers'
        ordering = ['time']

    def __unicode__(self):
        if self.title:
            return self.title
        else:
            return self.public_id


class MediaItemTranscoded(MediaResource):
    "Item file transcoded"

    element_type = 'transcoded item'

    item            = models.ForeignKey('MediaItem', related_name="transcoded", verbose_name=_('item'))
    mimetype        = models.CharField(_('mime_type'), max_length=255, blank=True)
    date_added      = DateTimeField(_('date'), auto_now_add=True)
    status          = models.IntegerField(_('status'), choices=ITEM_TRANSODING_STATUS, default=1)
    file            = models.FileField(_('file'), upload_to='items/%Y/%m/%d', max_length=1024, blank=True)

    @property
    def mime_type(self):
        if not self.mimetype:
            if self.file:
                if os.path.exists(self.file.path):
                    self.mimetype = mimetypes.guess_type(self.file.path)[0]
                    self.save()
                    return self.mimetype
                else:
                    return 'none'
            else:
                return 'none'
        else:
            return self.mimetype

    def __unicode__(self):
        if self.item.title:
            return self.item.title + ' - ' + self.mime_type
        else:
            return self.item.public_id + ' - ' + self.mime_type

    class Meta(MetaCore):
        db_table = app_name + '_media_transcoded'


class MediaItemTranscodingFlag(ModelCore):
    "Item flag to know if the MediaItem has been transcoded to a given format"

    item            = ForeignKey('MediaItem', related_name="transcoding", verbose_name=_('item'))
    mime_type       = CharField(_('mime_type'), required=True)
    date            = DateTimeField(_('date'), auto_now=True)
    value           = BooleanField(_('transcoded'))

    class Meta(MetaCore):
        db_table = 'media_transcoding'


class MediaItemIdentifier(Identifier):
    """Item identifier"""

    item = ForeignKey(MediaItem, related_name="identifiers", verbose_name=_('item'))

    class Meta(MetaCore):
        db_table = 'media_item_identifier'
        verbose_name = _('item identifier')
        verbose_name_plural = _('item identifiers')
        unique_together = ('identifier', 'item')


class MediaPart(MediaResource):
    "Describe an item part"
    element_type = 'part'
    item  = ForeignKey('MediaItem', related_name="parts", verbose_name=_('item'))
    title = CharField(_('title'), required=True)
    start = FloatField(_('start'), required=True)
    end   = FloatField(_('end'), required=True)

    class Meta(MetaCore):
        db_table = 'media_parts'
        verbose_name = _('item part')

    def __unicode__(self):
        return self.title



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url, include
from django.http import HttpResponse
import os

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

js_info_dict = {
    'packages': ('telemeta',),
}

PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
robots_rules = open(PROJECT_ROOT + os.sep + 'robots.txt', 'r').read()

urlpatterns = patterns('',
    # Example:
    # (r'^sandbox/', include('sandbox.foo.urls')),

    # Uncomment the admin/doc line below and add 'django.contrib.admindocs'
    # to INSTALLED_APPS to enable admin documentation:
    # (r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    (r'^admin/django/', include(admin.site.urls)),
    #(r'^grappelli/', include('grappelli.urls')), # grappelli URLS

    # Telemeta
    (r'^', include('telemeta.urls')),

    # Languages
    (r'^i18n/', include('django.conf.urls.i18n')),
    (r'^jsi18n/$', 'django.views.i18n.javascript_catalog', js_info_dict),
    (r'^robots\.txt$', lambda r: HttpResponse(robots_rules, mimetype="text/plain")),

    )
